from __future__ import annotations

from typing import TYPE_CHECKING

from django import forms

from edc_sites import site_sites
from edc_utils import formatted_date, to_local

from ..consent_definition import ConsentDefinition
from ..exceptions import (
    ConsentDefinitionDoesNotExist,
    ConsentDefinitionNotConfiguredForUpdate,
    NotConsentedError,
)
from ..site_consents import site_consents

if TYPE_CHECKING:
    from ..stubs import ConsentLikeModel


__all__ = ["RequiresConsentModelFormMixin"]


class RequiresConsentModelFormMixin:
    """Model form mixin for CRF or PRN forms to access the consent.

    Use with CrfModelMixin, etc
    """

    def clean(self):
        cleaned_data = super().clean()
        consent_obj = self.validate_against_consent()
        self.validate_against_dob(consent_obj)
        return cleaned_data

    def validate_against_dob(self, consent_obj):
        if consent_obj and to_local(self.report_datetime).date() < consent_obj.dob:
            dte_str = formatted_date(consent_obj.dob)
            raise forms.ValidationError(f"Report datetime cannot be before DOB. Got {dte_str}")

    def validate_against_consent(self) -> ConsentLikeModel | None:
        """Raise an exception if the report datetime doesn't make
        sense relative to the consent.
        """
        consent_obj = None
        if self.report_datetime:
            try:
                consent_obj = site_consents.get_consent_or_raise(
                    subject_identifier=self.get_subject_identifier(),
                    report_datetime=self.report_datetime,
                    site_id=self.site.id,
                )
            except (NotConsentedError, ConsentDefinitionNotConfiguredForUpdate) as e:
                raise forms.ValidationError({"__all__": str(e)}) from e
        return consent_obj

    @property
    def consent_definition(self) -> ConsentDefinition:
        """Returns a consent_definition from the schedule"""
        schedule = getattr(self, "related_visit", self).schedule
        try:
            cdef = schedule.get_consent_definition(
                site=site_sites.get(self.site.id),
                report_datetime=self.report_datetime,
            )
        except ConsentDefinitionDoesNotExist as e:
            raise forms.ValidationError(e) from e
        return cdef



from django.utils.safestring import mark_safe

user_profile_fieldsets = (
    (
        "Profile details",
        (
            {
                "fields": (
                    "job_title",
                    "alternate_email",
                    "mobile",
                )
            }
        ),
    ),
    (
        "Roles",
        ({"fields": ("roles",)}),
    ),
    (
        "Multisite viewer status",
        (
            {
                "description": mark_safe(
                    "Multisite viewer status designates the user with view only access to "
                    "data from the current site and all other sites the user has "
                    "permission to access (see Sites below). <BR>"
                    "<BR><B>Note:</B> <i>This status is restricted to accounts that do "
                    "not have add/change/delete permissions to other objects.</i><br>"
                    "<BR>Allowing `multisite viewers` status is done in two steps. "
                    "<ol><li>Adjust the account permissions accordingly and save "
                    "this form <li>Re-open this form, select the option, "
                    "and save the form again.</ol>"
                ),  # nosec B703 B308
                "fields": ("is_multisite_viewer",),
            }
        ),
    ),
    (
        "Sites",
        ({"fields": ("sites",)}),
    ),
    (
        "Email and SMS",
        (
            {
                "fields": (
                    "email_notifications",
                    "sms_notifications",
                )
            }
        ),
    ),
    (
        "Data export",
        ({"fields": ("export_format",)}),
    ),
    (
        "Printing profile",
        (
            {
                "fields": (
                    "clinic_label_printer",
                    "lab_label_printer",
                    "print_server",
                )
            }
        ),
    ),
)



from __future__ import annotations

import warnings

from edc_auth.get_app_codenames import get_app_codenames

warnings.warn(
    "This path/func name is deprecated in favor of get_app_codenames.get_app_codenames.",
    DeprecationWarning,
    stacklevel=2,
)


def get_clinic_codenames(
    *crud_apps: str,
    list_app: str | None = None,
    autocomplete_models: list[str] | None = None,
):
    return get_app_codenames(
        *crud_apps,
        list_app=list_app,
        autocomplete_models=autocomplete_models,
    )



from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import (
    RANDO_BLINDED,
    RANDO_UNBLINDED,
    get_rando_permissions_codenames,
    make_randomizationlist_view_only,
    update_rando_group_permissions,
)


def update_site_auths() -> None:
    site_auths.add_post_update_func(
        "edc_randomization", remove_default_model_permissions_from_edc_permissions
    )

    site_auths.add_group(get_rando_permissions_codenames, name=RANDO_BLINDED, view_only=True)
    site_auths.add_group(get_rando_permissions_codenames, name=RANDO_UNBLINDED, view_only=True)
    site_auths.add_post_update_func("edc_randomization", update_rando_group_permissions)
    site_auths.add_post_update_func("edc_randomization", make_randomizationlist_view_only)


update_site_auths()



# Generated by Django 5.0.7 on 2024-08-12 16:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_qareports", "0007_qareportlog_edc_qarepor_accesse_738ffe_idx"),
    ]

    operations = [
        migrations.CreateModel(
            name="QaReportLogSummary",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("username", models.CharField(max_length=100)),
                ("report_model", models.CharField(max_length=100)),
                ("first_accessed", models.DateTimeField()),
                ("last_accessed", models.DateTimeField()),
                ("access_count", models.IntegerField()),
            ],
            options={
                "verbose_name": "QA Report Log Summary",
                "verbose_name_plural": "QA Report Log Summary",
                "db_table": "qa_report_log_summary_view",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
    ]



from django.conf import settings


def get_autodiscover():
    return getattr(settings, "EDC_NAVBAR_AUTODISCOVER", True)


def get_verify_on_load():
    return getattr(settings, "EDC_NAVBAR_VERIFY_ON_LOAD", "")


def get_register_default_navbar():
    return getattr(settings, "EDC_NAVBAR_REGISTER_DEFAULT_NAVBAR", True)


def get_default_navbar_name():
    """Returns the default navbar name.

    For example: inte_dashboard for project INTE.
    """
    return getattr(
        settings, "EDC_NAVBAR_DEFAULT_NAVBAR_NAME", f"{settings.APP_NAME}_dashboard".lower()
    )



from __future__ import annotations

from datetime import date, datetime
from zoneinfo import ZoneInfo

from django.utils import timezone


class EdcDatetimeError(Exception):
    pass


def get_utcnow() -> datetime:
    return timezone.localtime(None, timezone=ZoneInfo("UTC"))


def get_utcnow_as_date() -> date:
    return timezone.localtime(None, timezone=ZoneInfo("UTC")).date()


def to_utc(dte: datetime) -> datetime | None:
    """Returns UTC datetime from any aware datetime."""
    return timezone.localtime(dte, timezone=ZoneInfo("UTC")) if dte else None


def to_local(dte: datetime) -> datetime | None:
    """Returns local datetime from any aware datetime."""
    return timezone.localtime(dte) if dte else None


def floor_secs(dte) -> datetime:
    return datetime(
        dte.year, dte.month, dte.day, dte.hour, dte.minute, 0, 0, tzinfo=dte.tzinfo
    )


def ceil_secs(dte) -> datetime:
    return datetime(
        dte.year,
        dte.month,
        dte.day,
        dte.hour,
        dte.minute,
        59,
        999999,
        tzinfo=dte.tzinfo,
    )


def floor_datetime(dt) -> datetime:
    return dt.replace(hour=0, minute=0, second=0, microsecond=0, tzinfo=dt.tzinfo)


def ceil_datetime(dt) -> datetime:
    return dt.replace(hour=23, minute=59, second=59, microsecond=999999, tzinfo=dt.tzinfo)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.db import models
from django.db.models import PROTECT, UniqueConstraint

from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_utils.round_up import round_half_away_from_zero

from .formulation_type import FormulationType
from .medication import Medication
from .route import Route
from .units import Units

if TYPE_CHECKING:
    from ..medication import Assignment


class Manager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, name, strength, units, formulation_type):
        return self.get(name, strength, units, formulation_type)


class Formulation(BaseUuidModel):
    medication = models.ForeignKey(Medication, on_delete=PROTECT, null=True, blank=False)

    strength = models.DecimalField(max_digits=6, decimal_places=1)

    units = models.ForeignKey(Units, on_delete=PROTECT)

    formulation_type = models.ForeignKey(FormulationType, on_delete=PROTECT)

    route = models.ForeignKey(Route, on_delete=PROTECT)

    notes = models.TextField(max_length=250, default="", blank=True)

    description = models.CharField(max_length=250, default="", blank=True)

    imp = models.BooleanField(default=False)

    imp_description = models.CharField(max_length=250, default="", blank=True)

    objects = Manager()

    history = HistoricalRecords()

    def __str__(self):
        return self.description

    def natural_key(self):
        return (
            self.medication,
            self.strength,
            self.units,
            self.formulation_type,
        )

    def save(self, *args, **kwargs):
        self.description = self.get_description()
        if not self.imp:
            self.imp_description = ""
        else:
            self.imp_description = (
                self.imp_description if self.imp_description else self.description
            )
        super().save(*args, **kwargs)

    def get_description(self) -> str:
        return (
            f"{self.medication} {round_half_away_from_zero(self.strength, 0)}"
            f"{self.get_units_display()} "
            f"{self.get_route_display()}"
        )

    def get_product_description(self):
        return (
            f"{self.medication} {round_half_away_from_zero(self.strength, 0)}"
            f"{self.get_units_display()} "
        )

    def get_description_with_assignment(self, assignment: Assignment) -> str:
        description = self.description
        return (
            f"{self.medication.display_name.title()} "
            f"{assignment.display_name.upper()} "
            f"{description.split(str(self.medication))[1]}"
        )

    def get_formulation_type_display(self):
        return self.formulation_type.display_name

    def get_units_display(self):
        return self.units.display_name

    def get_route_display(self):
        return self.route.display_name

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Formulation"
        verbose_name_plural = "Formulations"
        constraints = (
            UniqueConstraint(
                fields=["medication", "strength", "units", "formulation_type"],
                name="%(app_label)s_%(class)s_med_stren_uniq",
            ),
        )



from django.contrib import admin

from edc_model_admin.history import SimpleHistoryAdmin

from ...admin_site import edc_pharmacy_admin
from ...forms import LocationForm
from ...models import Location
from ..model_admin_mixin import ModelAdminMixin


@admin.register(Location, site=edc_pharmacy_admin)
class LocationAdmin(ModelAdminMixin, SimpleHistoryAdmin):
    history_list_display = ()
    show_cancel = True
    ordering = ("name",)
    list_per_page = 20

    form = LocationForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "name",
                    "display_name",
                    "site",
                )
            },
        ),
        (
            "Contact",
            {
                "fields": (
                    "contact_name",
                    "contact_tel",
                    "contact_email",
                )
            },
        ),
    )

    search_fields = ("id", "name", "contact_name")

    def get_readonly_fields(self, request, obj=None):  # noqa: ARG002
        if obj:
            return tuple({*self.readonly_fields, "name", "display_name"})
        return self.readonly_fields



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple

from ..admin_site import edc_lab_admin
from ..forms import ManifestItemForm
from ..models import ManifestItem
from .base_model_admin import BaseModelAdmin


class ManifestItemInlineAdmin(admin.TabularInline):
    model = ManifestItem
    form = ManifestItemForm
    extra = 1


@admin.register(ManifestItem, site=edc_lab_admin)
class ManifestItemAdmin(BaseModelAdmin, admin.ModelAdmin):
    form = ManifestItemForm

    fieldsets = (
        (None, {"fields": ("manifest", "identifier", "comment")}),
        audit_fieldset_tuple,
    )

    list_display = ("identifier",)

    def get_list_display(self, request) -> tuple:
        list_display = super().get_list_display(request)
        return tuple(set(("identifier",) + list_display))



from django.apps import AppConfig as DjangoAppConfig
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_data_manager"
    verbose_name = "Data Management"
    description = ""
    admin_site_name = "edc_data_manager_admin"
    include_in_administration_section = True
    has_exportable_data = True
    default_auto_field = "django.db.models.BigAutoField"



from __future__ import annotations

from decimal import Decimal
from typing import TYPE_CHECKING

from ..exceptions import InsufficientStockError

if TYPE_CHECKING:
    from ..models import Stock


def get_unit_qty_out(stock: Stock) -> Decimal:
    """Get the unit_qty_out by summing the container qty per
    stock obj.
    """
    unit_qty_out = 0
    for stock_obj in stock.__class__.objects.filter(from_stock=stock):
        unit_qty_out += stock_obj.container.qty
    if stock.unit_qty_out > stock.unit_qty_in:
        raise InsufficientStockError(f"Unit QTY OUT cannot exceed Unit QTY IN. See {stock}.")
    return unit_qty_out



import pandas as pd
from clinicedc_constants import FEMALE, MALE

from ..constants import MEAN_95CI, N_ONLY, N_WITH_COL_PROP, N_WITH_ROW_PROP
from ..row import RowDefinition, RowDefinitions
from ..table import Table


class HbA1cTable(Table):
    def __init__(self, main_df: pd.DataFrame = None):
        super().__init__(
            colname="hba1c_value",
            main_df=main_df,
            title="HbA1C (%) categories",
        )

    @property
    def row_definitions(self) -> RowDefinitions:
        df_tmp = self.main_df.copy()
        row_defs = RowDefinitions(reverse_rows=False)
        row0 = RowDefinition(
            title=self.title,
            label=self.default_sublabel,
            condition=(df_tmp["gender"].notna()),
            columns={FEMALE: (N_ONLY, 2), MALE: (N_ONLY, 2), "All": (N_ONLY, 2)},
            drop=False,
        )
        row_defs.add(row0)

        columns = {
            FEMALE: (MEAN_95CI, 2),
            MALE: (MEAN_95CI, 2),
            "All": (MEAN_95CI, 2),
        }
        row_mean = RowDefinition(
            colname=self.colname,
            label="Mean (95% CI)",
            condition=(self.main_df[self.colname].notna()),
            columns=columns,
            drop=False,
        )

        columns = {
            FEMALE: (N_WITH_COL_PROP, 2),
            MALE: (N_WITH_COL_PROP, 2),
            "All": (N_WITH_ROW_PROP, 2),
        }
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="<6.0",
                condition=(self.main_df[self.colname] < 6.0),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="≥6.0 and ≤6.4",
                condition=(
                    (self.main_df[self.colname] >= 6.0) & (self.main_df[self.colname] <= 6.4)
                ),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label=">6.4",
                condition=(self.main_df[self.colname] > 6.4),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="not measured",
                condition=(self.main_df[self.colname].isna()),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(row_mean)
        return row_defs



from .crf import VisitScheduleCrfModelFormMixin
from .off_schedule_modelform_mixin import OffScheduleModelFormMixin
from .visit_schedule_non_crf_modelform_mixin import VisitScheduleNonCrfModelFormMixin

__all__ = [
    "OffScheduleModelFormMixin",
    "VisitScheduleCrfModelFormMixin",
    "VisitScheduleNonCrfModelFormMixin",
]



from __future__ import annotations

from typing import TYPE_CHECKING

from clinicedc_constants import CANCELLED, CLOSED, HIGH_PRIORITY, NEW, OPEN
from django import template

from edc_auth.utils import get_user
from edc_utils.date import to_local
from edc_utils.text import formatted_date

from ..models import ActionItem
from ..site_action_items import site_action_items
from ..utils import (
    get_parent_reference_obj,
    get_reference_obj,
    get_related_reference_obj,
)
from ..view_utils import ActionItemButton, ActionItemPopoverListItem

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest

    from edc_appointment.models import Appointment

    from ..action import Action


register = template.Library()


@register.inclusion_tag(
    "edc_action_item/action_item_button.html",
    takes_context=True,
)
def render_action_item_button(
    context,
    action_item: ActionItem | None = None,
    name: str | None = None,
    label: str | None = None,
    color: str | None = None,
    appointment: Appointment = None,
) -> dict[str, ActionItemButton]:
    if action_item:
        action_cls = action_item.action_cls
    else:
        action_cls = site_action_items.get(name)
    btn = ActionItemButton(
        subject_identifier=context["subject_identifier"],
        action_cls=action_cls,
        model_obj=action_item,
        model_cls=ActionItem,
        request=context["request"],
        user=context["request"].user,
        current_site=context["request"].site,
        appointment=appointment or context.get("appointment"),
        fixed_label=label,
        fixed_color=color,
    )
    return dict(btn=btn)


@register.inclusion_tag(
    "edc_action_item/add_action_item_popover.html",
    takes_context=True,
)
def add_action_item_popover(
    context,
    subject_identifier: str = None,
    subject_dashboard_url: str = None,
    color: str = None,
) -> dict[str, str | Action | WSGIRequest | Appointment]:
    """
    `Add action linked PRN forms`
    """
    action_item_add_url: str = "edc_action_item_admin:edc_action_item_actionitem_add"
    add_actions: dict[str, type[Action]] = site_action_items.get_add_actions_to_show()
    if add_actions:
        d = {k: v.display_name for k, v in add_actions.items()}
        add_actions = {}
        for k in sorted(d, key=d.get):
            add_actions.update({k: d[k]})
    return dict(
        action_item_add_url=action_item_add_url,
        subject_identifier=subject_identifier,
        subject_dashboard_url=subject_dashboard_url,
        add_actions=add_actions,
        request=context["request"],
        appointment=context.get("appointment"),
        color=color,
    )


@register.inclusion_tag(
    "edc_action_item/action_item_with_popover.html",
    takes_context=True,
)
def action_item_with_popover(context, action_item: ActionItem, tabindex):
    date_last_updated = None
    user_last_updated = None
    if reference_obj := get_reference_obj(action_item):
        date_last_updated = formatted_date(
            to_local(reference_obj.modified or reference_obj.created)
        )
        if get_user(reference_obj.user_modified or reference_obj.user_created):
            user_last_updated = reference_obj.user_modified or reference_obj.user_created
        else:
            user_last_updated = None

    return dict(
        action_item=action_item,
        action_identifier=action_item.action_identifier,
        subject_identifier=context.get("subject_identifier"),
        action_item_model_name="edc_action_item.actionitem",
        next_url_name="subject_dashboard_url",
        request=context["request"],
        appointment=context.get("appointment"),
        report_datetime=formatted_date(to_local(action_item.report_datetime)),
        date_last_updated=date_last_updated,
        user_last_updated=user_last_updated,
        CANCELLED=CANCELLED,
        CLOSED=CLOSED,
        HIGH_PRIORITY=HIGH_PRIORITY,
        NEW=NEW,
        OPEN=OPEN,
        tabindex=tabindex,
        popover_title=ActionItem._meta.verbose_name,
        # priority=
    )


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/popover_list_item.html",
    takes_context=True,
)
def render_popover_list_item(
    context,
    action_item: ActionItem = None,
    action_item_type: str = None,
    appointment: Appointment | None = None,
) -> dict:
    """Renders one ListItem"""
    opts = dict(
        action_item=action_item,
        category=action_item_type,
        subject_identifier=context.get("subject_identifier"),
        next_url_name=context.get("next_url_name"),
        user=context["request"].user,
        current_site=context["request"].site,
        request=context["request"],
        appointment=appointment,
    )
    btn = ActionItemPopoverListItem(**opts)
    return dict(btn=btn)


@register.inclusion_tag(
    "edc_action_item/action_item_reason.html",
)
def render_action_item_reason(action_item):
    action_item_reasons = []
    objects = [
        get_reference_obj(action_item),
        get_parent_reference_obj(action_item),
        get_related_reference_obj(action_item),
    ]
    for obj in objects:
        try:
            action_item_reasons.append(obj.get_action_item_reason())
        except AttributeError as e:
            if "get_action_item_reason" not in str(e):
                raise
    if action_item_reasons:
        action_item_reasons = list(set(action_item_reasons))
    return {"action_item_reasons": action_item_reasons}



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.core.handlers.wsgi import WSGIRequest
from django.db import transaction
from django.utils import timezone

from ..exceptions import ConfirmAtLocationError

if TYPE_CHECKING:
    from uuid import UUID

    from ..models import (
        ConfirmationAtLocation,
        ConfirmationAtLocationItem,
        Location,
        Stock,
        StockTransfer,
        StockTransferItem,
    )


def confirm_stock_at_location(
    stock_transfer: StockTransfer,
    stock_codes: list[str],
    location: UUID,
    request: WSGIRequest | None = None,
) -> tuple[list[str], list[str], list[str]]:
    """Confirm stock instances given a list of stock codes
    and a request/receive pk.

    Called from ConfirmStock view.

    See also: confirm_stock_action
    """
    confirmed_by = request.user.username
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    stock_transfer_item_model_cls: type[StockTransferItem] = django_apps.get_model(
        "edc_pharmacy.stocktransferitem"
    )
    confirm_at_location_model_cls: type[ConfirmationAtLocation] = django_apps.get_model(
        "edc_pharmacy.confirmationatlocation"
    )
    confirmation_at_location_item_model_cls: type[ConfirmationAtLocationItem] = (
        django_apps.get_model("edc_pharmacy.confirmationatlocationitem")
    )
    location_model_cls: type[Location] = django_apps.get_model("edc_pharmacy.location")

    location = location_model_cls.objects.get(pk=location)

    confirm_at_location, _ = confirm_at_location_model_cls.objects.get_or_create(
        stock_transfer=stock_transfer,
        location=location,
    )

    confirmed_codes, already_confirmed_codes, invalid_codes = [], [], []

    stock_codes = [s.strip() for s in stock_codes]
    valid_codes = [obj.code for obj in stock_model_cls.objects.filter(code__in=stock_codes)]
    invalid_codes = [c for c in stock_codes if c not in valid_codes]
    already_confirmed_codes = [
        obj.code
        for obj in stock_model_cls.objects.filter(
            stocktransferitem__stock_transfer=stock_transfer,
            stocktransferitem__confirmationatlocationitem__isnull=False,
        )
        if obj.code in valid_codes
    ]
    for stock_code in [c for c in valid_codes if c not in already_confirmed_codes]:
        stock_transfer_item = stock_transfer_item_model_cls.objects.get(
            stock__code=stock_code, stock_transfer=stock_transfer
        )
        obj = confirmation_at_location_item_model_cls(
            confirm_at_location=confirm_at_location,
            # stock=stock,
            code=stock_code,
            stock_transfer_item=stock_transfer_item,
            confirmed_datetime=timezone.now(),
            confirmed_by=confirmed_by,
            user_created=confirmed_by,
            created=timezone.now(),
        )
        try:
            obj.save()
        except ConfirmAtLocationError as e:
            messages.add_message(request, messages.ERROR, str(e))
            invalid_codes.append(stock_code)
        else:
            confirmed_codes.append(stock_code)
    return confirmed_codes, already_confirmed_codes, invalid_codes


__all__ = ["confirm_stock_at_location"]



# Generated by Django 5.2.8 on 2025-11-13 05:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_export", "0024_alter_datarequest_revision_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="ExportReceipt",
        ),
        migrations.DeleteModel(
            name="FileHistory",
        ),
        migrations.RemoveField(
            model_name="historicalexportreceipt",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalfilehistory",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalobjecthistory",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalplan",
            name="history_user",
        ),
        migrations.DeleteModel(
            name="ObjectHistory",
        ),
        migrations.DeleteModel(
            name="Plan",
        ),
        migrations.DeleteModel(
            name="UploadExportReceiptFile",
        ),
        migrations.DeleteModel(
            name="HistoricalExportReceipt",
        ),
        migrations.DeleteModel(
            name="HistoricalFileHistory",
        ),
        migrations.DeleteModel(
            name="HistoricalObjectHistory",
        ),
        migrations.DeleteModel(
            name="HistoricalPlan",
        ),
    ]



from .constants import PER_DAY
from .dosage_calculator import DosageCalculator, DosageError


def dosage_per_day(dosage_guideline=None, **kwargs) -> float:
    if dosage_guideline.frequency_units.name != PER_DAY:
        raise DosageError(
            "Dosage is not calculated per day. Got dosage_guideline.frequency_units="
            f"{dosage_guideline.frequency_units.name}"
        )
    return DosageCalculator(dosage_guideline=dosage_guideline, **kwargs).dosage



from textwrap import wrap

from clinicedc_constants import (
    CLOSED,
    FEEDBACK,
    HIGH_PRIORITY,
    NEW,
    NO,
    OPEN,
    RESOLVED,
    YES,
)
from django.conf import settings
from django.contrib import admin
from django.contrib.admin.decorators import register
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_audit_fields.admin import audit_fieldset_tuple

from edc_action_item.fieldsets import action_fields, action_fieldset_tuple
from edc_appointment.models import Appointment
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin
from edc_utils.text import formatted_datetime

from ..admin_site import edc_data_manager_admin
from ..auth_objects import DATA_MANAGER
from ..constants import CLOSED_WITH_ACTION
from ..forms import DataQueryForm
from ..models import DataDictionary, DataQuery
from .actions import toggle_dm_status


@register(DataQuery, site=edc_data_manager_admin)
class DataQueryAdmin(SiteModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    status_column_template_name = "edc_data_manager/columns/status.html"
    query_date_column_template_name = "edc_data_manager/columns/query_date.html"
    query_text_column_template_name = "edc_data_manager/columns/query_text.html"
    query_recipients_column_template_name = "edc_data_manager/columns/query_recipients.html"

    rule_generated_column_template_name = "edc_data_manager/columns/rule_generated.html"

    locked_column_template_name = "edc_data_manager/columns/locked.html"

    status_column_context = {  # noqa: RUF012
        "NEW": NEW,
        "OPEN": OPEN,
        "FEEDBACK": FEEDBACK,
        "RESOLVED": RESOLVED,
        "CLOSED": CLOSED,
        "CLOSED_WITH_ACTION": CLOSED_WITH_ACTION,
    }

    show_cancel = True
    show_object_tools = True

    form = DataQueryForm

    actions = (toggle_dm_status,)

    radio_fields = {  # noqa: RUF012
        "status": admin.VERTICAL,
        "site_response_status": admin.VERTICAL,
        "query_priority": admin.VERTICAL,
    }

    autocomplete_fields = (
        "sender",
        "recipients",
        "data_dictionaries",
        "requisition_panel",
        "visit_schedule",
        "registered_subject",
        "dm_user",
    )

    list_display = (
        "wrapped_title",
        "subject",
        "query_status",
        "sent_to",
        "query",
        "generated",
        "lock",
        "created",
        "modified",
        "reference",
    )

    list_filter = (
        "site_response_status",
        "status",
        "query_priority",
        "report_datetime",
        "site_resolved_datetime",
        "resolved_datetime",
        "rule_generated",
        "auto_resolved",
        "missed_visit",
        "locked",
        "title",
        "created",
        "modified",
        "site",
    )

    search_fields = (
        "title",
        "action_identifier",
        "sender__first_name",
        "sender__last_name",
        "sender__email",
        "sender__username",
        "recipients__first_name",
        "recipients__last_name",
        "recipients__email",
        "recipients__username",
        "registered_subject__subject_identifier",
        "data_dictionaries__model_verbose_name",
        "data_dictionaries__prompt",
        "data_dictionaries__field_name",
        "data_dictionaries__number",
        "requisition_panel__name",
        "requisition_panel__display_name",
        "query_text",
        "site_response_text",
        "plan_of_action",
    )

    fieldsets = (
        [
            None,
            {
                "fields": (
                    "registered_subject",
                    "report_datetime",
                    "sender",
                    "query_priority",
                    "recipients",
                    "rule_generated",
                )
            },
        ],
        [
            "Query Details",
            {
                "fields": (
                    "title",
                    "visit_schedule",
                    "visit_code_sequence",
                    "data_dictionaries",
                    "requisition_panel",
                    "query_text",
                )
            },
        ],
        [
            "Site Response",
            {
                "fields": (
                    "site_response_status",
                    "site_response_text",
                    "site_resolved_datetime",
                    "missed_visit",
                    "auto_resolved",
                )
            },
        ],
        [
            "For Data Manager Only",
            {
                "fields": (
                    "status",
                    "resolved_datetime",
                    "dm_user",
                    "plan_of_action",
                    "locked",
                    "locked_reason",
                )
            },
        ],
        [
            "Rules",
            {"classes": ("collapse",), "fields": ("rule_reference",)},
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    def wrapped_title(self, obj=None):
        wrapped_title = wrap(obj.title, width=20)
        html = "<BR>".join(wrapped_title)
        return format_html(
            "{}",
            mark_safe(html),  # nosec B703, B308
        )

    wrapped_title.short_description = "Title"

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == "data_dictionaries" and request.GET.get("title"):
            kwargs["queryset"] = DataDictionary.objects.filter(
                model_verbose_name=request.GET.get("title")
            )
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    def query_dates(self, obj):
        context = dict(
            report_datetime=formatted_datetime(
                obj.report_datetime, settings.SHORT_DATE_FORMAT
            ),
            site_resolved_datetime=formatted_datetime(
                obj.site_resolved_datetime, settings.SHORT_DATE_FORMAT
            ),
            resolved_datetime=formatted_datetime(
                obj.resolved_datetime, settings.SHORT_DATE_FORMAT
            ),
        )
        return self.render_query_date_to_string(context)

    def subject(self, obj):
        return self.dashboard(obj=obj, label=obj.subject_identifier)

    def query(self, obj):
        context = dict(
            plane_of_action=obj.plan_of_action,
            report_datetime=formatted_datetime(
                obj.report_datetime, settings.SHORT_DATE_FORMAT
            ),
            site_resolved_datetime=formatted_datetime(
                obj.site_resolved_datetime, settings.SHORT_DATE_FORMAT
            ),
            resolved_datetime=formatted_datetime(
                obj.resolved_datetime, settings.SHORT_DATE_FORMAT
            ),
            visit_schedule=obj.visit_schedule,
            questions=obj.data_dictionaries.all().order_by("model", "number"),
            requisition_panel=obj.requisition_panel,
            site_response_text=obj.site_response_text,
            status=obj.status,
            dm_user=obj.dm_user,
            query_text=obj.query_text,
            query_priority_display=obj.get_query_priority_display(),
            query_priority=obj.query_priority,
            HIGH_PRIORITY=HIGH_PRIORITY,
            RESOLVED=RESOLVED,
            CLOSED=CLOSED,
            CLOSED_WITH_ACTION=CLOSED_WITH_ACTION,
        )
        return self.render_query_text_to_string(context)

    @staticmethod
    def dm(obj):
        return obj.sender.first_name

    def generated(self, obj):
        rule_generated = YES if obj.rule_generated else NO
        context = {"rule_generated": rule_generated, "YES": YES, "NO": NO}
        return self.render_rule_generated_to_string(context)

    def sent_to(self, obj):
        context = {"recipients": obj.recipients.all()}
        return self.render_query_recipients_to_string(context)

    @staticmethod
    def resolved_by(obj):
        if obj.dm_user:
            return f"{obj.dm_user.first_name} {obj.dm_user.last_name}"
        return None

    @staticmethod
    def reference(obj):
        return obj.action_identifier[-9:]

    def query_status(self, obj):
        context = {
            "site_response_status": obj.site_response_status,
            "dm_status": obj.status,
            "site_response_status_display": obj.get_site_response_status_display(),
            "dm_status_display": obj.get_status_display(),
        }
        return self.render_status_to_string(context)

    query_status.short_description = "Status"

    def lock(self, obj):
        context = {"locked": obj.locked}
        return self.render_locked_to_string(context)

    lock.short_description = "Locked"

    def render_status_to_string(self, context):
        context.update(self.status_column_context)
        return render_to_string(self.status_column_template_name, context=context)

    def render_locked_to_string(self, context):
        return render_to_string(self.locked_column_template_name, context=context)

    def render_rule_generated_to_string(self, context):
        return render_to_string(self.rule_generated_column_template_name, context=context)

    def render_query_date_to_string(self, context):
        return render_to_string(self.query_date_column_template_name, context=context)

    def render_query_text_to_string(self, context):
        return render_to_string(self.query_text_column_template_name, context=context)

    def render_query_recipients_to_string(self, context):
        return render_to_string(self.query_recipients_column_template_name, context=context)

    def created_details(self, obj):
        rule_generated = YES if obj.rule_generated else NO
        context = {
            "rule_generated": rule_generated,
            "YES": YES,
            "NO": NO,
            "created": obj.created,
            "modified": obj.modified,
            "reference": obj.action_identifier[-9:],
        }
        return render_to_string(self.query_date_column_template_name, context=context)

    def get_readonly_fields(self, request, obj=None) -> tuple:
        fields = super().get_readonly_fields(request, obj=obj)
        extra_fields = (
            "rule_generated",
            "rule_reference",
            "missed_visit",
            "auto_resolved",
            "registered_subject",
            *action_fields,
        )
        if not request.user.groups.filter(name=DATA_MANAGER):
            extra_fields = (
                "data_dictionaries",
                "dm_user",
                "plan_of_action",
                "query_priority",
                "query_text",
                "registered_subject",
                "recipients",
                "report_datetime",
                "requisition_panel",
                "resolved_datetime",
                "sender",
                "status",
                "title",
                "visit_code_sequence",
                "visit_schedule",
                *extra_fields,
            )
        if not obj:
            extra_fields = tuple(f for f in extra_fields if f != "registered_subject")
        return *fields, *extra_fields

    def get_subject_dashboard_url_kwargs(self, obj):
        def get_opts():
            return dict(
                subject_identifier=obj.subject_identifier,
                visit_schedule_name=obj.visit_schedule.visit_schedule_name,
                schedule_name=obj.visit_schedule.schedule_name,
                visit_code=obj.visit_schedule.visit_code,
                visit_code_sequence=obj.visit_code_sequence or 0,
            )

        try:
            appointment = Appointment.objects.get(**get_opts())
        except (ObjectDoesNotExist, AttributeError):
            kwargs = dict(subject_identifier=obj.subject_identifier)
        else:
            kwargs = dict(
                subject_identifier=obj.subject_identifier,
                appointment=str(appointment.pk),
            )
        return kwargs

    def get_cancel_redirect_url(self, request=None, obj=None, **kwargs):
        return self.get_changelist_url(request=request, obj=obj, **kwargs)



# Generated by Django 6.0 on 2026-01-14 15:00

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0095_auto_20260113_2005"),
    ]

    operations = []



from .base_listboard_view import BaseListboardView


class ResultListboardView(BaseListboardView):
    form_action_url = "aliquot_form_action_url"
    listboard_template = "result_listboard_template"
    listboard_url = "result_listboard_url"
    listboard_model = "edc_lab.result"
    listboard_view_permission_codename = "edc_lab_dashboard.view_lab_result_listboard"
    listboard_view_only_my_permission_codename = None
    navbar_selected_item = "result"
    search_form_url = "result_listboard_url"



# Generated by Django 5.2.8 on 2025-11-20 23:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_metadata", "0035_crfmetadata_edc_metadata_crfmetadata_a4idx_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="requisitionmetadata",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "ordering": (
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                    "show_order",
                ),
                "verbose_name": "Requisition collection status",
                "verbose_name_plural": "Requisition collection status",
            },
        ),
        migrations.RemoveIndex(
            model_name="requisitionmetadata",
            name="edc_metadat_site_id_cc31cf_idx",
        ),
        migrations.RemoveIndex(
            model_name="requisitionmetadata",
            name="edc_metadat_subject_04cc77_idx",
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                    "show_order",
                ],
                name="edc_metadat_subject_5a33a7_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(fields=["site"], name="edc_metadat_site_id_eb923f_idx"),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=["entry_status"], name="edc_metadat_entry_s_c6264a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(fields=["model"], name="edc_metadat_model_29f7e8_idx"),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=["panel_name"], name="edc_metadat_panel_n_0de2e0_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=["due_datetime"], name="edc_metadat_due_dat_8ac98a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=["fill_datetime"], name="edc_metadat_fill_da_0dd40c_idx"
            ),
        ),
    ]



from django.contrib.admin import SimpleListFilter
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site


class SitesListFilter(SimpleListFilter):
    title = "Site"
    parameter_name = "site_name"

    def lookups(self, request, model_admin):  # noqa: ARG002
        sites = [
            (site.name, site.name.replace("_", " ").title())
            for site in Site.objects.all().order_by("name")
        ]
        return tuple(sites)

    def queryset(self, request, queryset):  # noqa: ARG002
        """Returns a queryset if the site name is in the list of sites"""
        qs = None
        if self.value():
            qs = get_user_model().objects.filter(userprofile__sites__name__in=[self.value()])
        return qs


class CountriesListFilter(SimpleListFilter):
    title = "Country"
    parameter_name = "country_name"

    def lookups(self, request, model_admin):  # noqa: ARG002
        countries = set(s.siteprofile.country for s in Site.objects.all())
        return tuple((c, c.replace("_", " ").title()) for c in sorted(countries))

    def queryset(self, request, queryset):  # noqa: ARG002
        """Returns a queryset if the country name is in a site's siteprofile,
        in the list of sites.
        """
        qs = None
        if self.value():
            qs = (
                get_user_model()
                .objects.filter(userprofile__sites__siteprofile__country__in=[self.value()])
                .distinct()
            )
        return qs



import sys

from django.apps import AppConfig as DjangoAppConfig
from django.conf import settings
from django.core.checks import register
from django.core.management.color import color_style
from django.db.backends.signals import connection_created

from edc_utils.sqlite import activate_foreign_keys

from .system_checks import check_for_edc_model

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_model"
    verbose_name = "Edc Model"

    def ready(self):
        register(check_for_edc_model)
        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        connection_created.connect(activate_foreign_keys)
        sys.stdout.write(f" * default TIME_ZONE {settings.TIME_ZONE}.\n")
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")



from django.db import models

from ..choices import REPORT_STATUS
from ..validators import datetime_not_future


class ReportStatusModelMixin(models.Model):
    report_status = models.CharField(
        verbose_name="What is the status of this report?",
        max_length=25,
        choices=REPORT_STATUS,
    )

    report_closed_datetime = models.DateTimeField(
        blank=True,
        null=True,
        validators=[datetime_not_future],
        verbose_name="Date and time report closed.",
    )

    class Meta:
        abstract = True



from __future__ import annotations

import pandas as pd
from django_pandas.io import read_frame
from edc_visit_schedule.models import SubjectScheduleHistory

from ...models import Stock, StorageBinItem
from ...utils import get_imp_schedule_names
from .get_next_scheduled_visit_for_subjects_df import (
    get_next_scheduled_visit_for_subjects_df,
)


def remove_exact_duplicates(s):
    seen = set()
    result = []
    for item in s.split(","):
        if item not in seen:
            seen.add(item)
            result.append(item)
    return ",".join(result)


def stock_for_subjects_df() -> pd.DataFrame:
    visit_schedule_names, schedule_names = get_imp_schedule_names()

    df_schedule = read_frame(
        SubjectScheduleHistory.objects.values(
            "subject_identifier",
            "visit_schedule_name",
            "schedule_name",
            "offschedule_datetime",
        ).filter(
            visit_schedule_name__in=visit_schedule_names,
            schedule_name__in=schedule_names,
            offschedule_datetime__isnull=True,
        )
    )
    df_stock_on_site = read_frame(
        Stock.objects.values(
            "code", "allocation__registered_subject__subject_identifier"
        ).filter(confirmationatlocationitem__isnull=False, dispenseitem__isnull=True),
        verbose=False,
    ).rename(
        columns={"allocation__registered_subject__subject_identifier": "subject_identifier"}
    )

    df = pd.merge(
        df_schedule[["subject_identifier", "offschedule_datetime"]],
        df_stock_on_site,
        on="subject_identifier",
        how="left",
    ).fillna("")

    df_storage = read_frame(
        StorageBinItem.objects.values("stock__code", "storage_bin__name").filter(
            stock__code__in=df["code"].tolist(),
            storage_bin__in_use=True,
            stock__dispenseitem__isnull=True,
        )
    ).rename(columns={"stock__code": "code", "storage_bin__name": "bin"})

    df = df.merge(df_storage[["code", "bin"]], on="code", how="left").fillna("")

    df_codes = (
        df.groupby("subject_identifier")["code"]
        .agg(",".join)
        .reset_index()
        .sort_values(by=["subject_identifier"])
        .reset_index(drop=True)
    )

    df_bins = (
        df.groupby("subject_identifier")["bin"]
        .agg(",".join)
        .reset_index()
        .sort_values(by=["subject_identifier"])
        .reset_index(drop=True)
    )

    df = df_codes.merge(df_bins, on="subject_identifier", how="left")

    df_appt = get_next_scheduled_visit_for_subjects_df()
    df_appt = (
        df_appt[
            [
                "subject_identifier",
                "site_id",
                "visit_code",
                "appt_datetime",
                "baseline_datetime",
            ]
        ]
        .copy()
        .reset_index(drop=True)
    )

    df = df.merge(df_appt, how="left", on="subject_identifier")
    df = df[(df.appt_datetime.notna())].reset_index(drop=True)

    utc_now = pd.Timestamp.utcnow().tz_localize(None)
    df["relative_days"] = (df.appt_datetime - utc_now).dt.days
    # df = df[(df.relative_days >= -105)]
    df["appt_date"] = df.appt_datetime.dt.date
    df.loc[df["code"].str.strip() == "", "code"] = pd.NA
    df.loc[df["bin"].str.strip() == "", "bin"] = pd.NA
    df.loc[~df["bin"].isna(), "bin"] = df.loc[~df["bin"].isna(), "bin"].apply(
        remove_exact_duplicates
    )
    df = df.rename(columns={"code": "codes", "bin": "bins"}).reset_index(drop=True)
    return df



# Generated by Django 4.1.2 on 2022-11-29 02:24

import uuid

import django.db.models.deletion
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="DiagnosisLocations",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("version", models.CharField(default="1.0", editable=False, max_length=35)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Diagnosis Locations",
                "verbose_name_plural": "Diagnosis Locations",
                "ordering": ["display_index", "display_name"],
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
        migrations.CreateModel(
            name="HistoricalDiagnosisLocations",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("version", models.CharField(default="1.0", editable=False, max_length=35)),
                ("id", models.IntegerField(blank=True, db_index=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Diagnosis Locations",
                "verbose_name_plural": "historical Diagnosis Locations",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalReasonsForTesting",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("version", models.CharField(default="1.0", editable=False, max_length=35)),
                ("id", models.IntegerField(blank=True, db_index=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Reasons for Testing",
                "verbose_name_plural": "historical Reasons for Testing",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="ReasonsForTesting",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("version", models.CharField(default="1.0", editable=False, max_length=35)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Reasons for Testing",
                "verbose_name_plural": "Reasons for Testing",
                "ordering": ["display_index", "display_name"],
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
        migrations.AddIndex(
            model_name="reasonsfortesting",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="edc_dx_revi_id_9cf36d_idx",
            ),
        ),
        migrations.AddField(
            model_name="historicalreasonsfortesting",
            name="history_user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="historicaldiagnosislocations",
            name="history_user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddIndex(
            model_name="diagnosislocations",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="edc_dx_revi_id_ab6550_idx",
            ),
        ),
    ]



from __future__ import annotations

from clinicedc_constants import NOT_APPLICABLE, OTHER
from django.conf import settings
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_utils.text import convert_php_dateformat

from ..forms import AeTmgForm
from ..models import AeClassification
from ..utils import get_adverse_event_app_label, get_ae_model
from .modeladmin_mixins import NonAeInitialModelAdminMixin


class AeTmgModelAdminMixin(
    ModelAdminSubjectDashboardMixin,
    NonAeInitialModelAdminMixin,
    ActionItemModelAdminMixin,
):
    form = AeTmgForm

    additional_instructions = "For completion by TMG Investigators Only"

    search_fields = (
        "subject_identifier",
        "action_identifier",
        "ae_initial__action_identifier",
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "ae_initial", "report_datetime")}),
        (
            "Original Report",
            {
                "fields": (
                    "ae_description",
                    "ae_classification",
                    "ae_classification_other",
                )
            },
        ),
        (
            "Investigator's section",
            {
                "fields": (
                    "ae_received_datetime",
                    "clinical_review_datetime",
                    "investigator_comments",
                    "original_report_agreed",
                    "investigator_narrative",
                    "investigator_ae_classification",
                    "investigator_ae_classification_other",
                    "officials_notified",
                    "report_status",
                    "report_closed_datetime",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {  # noqa: RUF012
        "report_status": admin.VERTICAL,
        "original_report_agreed": admin.VERTICAL,
        "investigator_ae_classification": admin.VERTICAL,
    }

    def get_list_display(self, request) -> tuple[str]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "status",
            "ae_initial",
            "report_datetime",
            "officials_notified",
            "report_closed_datetime",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("report_datetime", "report_status")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    @staticmethod
    def status(obj=None):
        return obj.report_status.title()

    def get_queryset(self, request):
        """Returns for the current user if has `view_aetmg` permissions."""
        # TODO: this used to look at group membership?
        if request.user.has_perm(f"{get_adverse_event_app_label()}.view_aetmg"):
            return super().get_queryset(request).all()
        return super().get_queryset(request)

    def get_changeform_initial_data(self, request):
        """Updates initial data with the description of the
        original AE.
        """
        initial = super().get_changeform_initial_data(request)
        ae_initial_model_cls = get_ae_model("aeinitial")
        try:
            ae_initial = ae_initial_model_cls.objects.get(pk=request.GET.get("ae_initial"))
        except ObjectDoesNotExist:
            pass
        else:
            try:
                ae_classification = ae_initial.ae_classification.name
            except AttributeError:
                ae_classification = None
            else:
                if ae_initial.ae_classification.name == OTHER:
                    other = ae_initial.ae_classification_other.rstrip()
                    ae_classification = f"{ae_classification}: {other}"
            report_datetime = ae_initial.report_datetime.strftime(
                convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
            )
            initial.update(
                ae_classification=ae_classification,
                ae_description=f"{ae_initial.ae_description} (reported: {report_datetime})",
                investigator_ae_classification=AeClassification.objects.get(
                    name=NOT_APPLICABLE
                ).id,
            )
        return initial



from __future__ import annotations

from clinicedc_constants import EQ
from django.core.validators import MinValueValidator
from django.db import models

from edc_constants.choices import GRADING_SCALE_WITH_NOT_GRADED, YES_NO
from edc_lab.choices import RESULT_QUANTIFIER
from edc_reportable.choices import REPORTABLE


def get_field_attrs_for_utestid(
    utest_id: str,
    units_choices: tuple,
    default_units: str | None = None,
    verbose_name: str | None = None,
    decimal_places: int | None = None,
    max_digits: int | None = None,
    validators: list | None = None,
    quantifier: list | None = None,
    help_text: list | None = None,
) -> dict:
    """Returns a dictionary of field classes for the model"""
    value_options = dict(
        verbose_name=verbose_name or utest_id.upper(),
        decimal_places=decimal_places if decimal_places is not None else 2,
        max_digits=max_digits if max_digits is not None else 8,
        validators=validators or [MinValueValidator(0.00)],
        null=True,
        blank=True,
    )
    if help_text:
        value_options.update(help_text=help_text)
    units_options = dict(
        verbose_name="units",
        max_length=15,
        choices=units_choices,
        null=True,
        blank=True,
    )
    if default_units:
        units_options.update(default=default_units)

    quantifier_options = dict(
        verbose_name="Quantifier",
        max_length=10,
        choices=RESULT_QUANTIFIER,
        default=EQ,
        null=True,
        blank=True,
    )
    return {
        f"{utest_id}_value": models.DecimalField(**value_options),
        f"{utest_id}_units": models.CharField(**units_options),
        f"{utest_id}_quantifier": models.CharField(**quantifier_options),
    }


def get_field_attrs_for_reportable(utest_id: str) -> dict:
    """Returns a dictionary of field classes for the model"""
    return {
        f"{utest_id}_abnormal": models.CharField(
            verbose_name="abnormal",
            choices=YES_NO,
            max_length=25,
            null=True,
            blank=True,
        ),
        f"{utest_id}_reportable": models.CharField(
            verbose_name="reportable",
            choices=REPORTABLE,
            max_length=25,
            null=True,
            blank=True,
        ),
        f"{utest_id}_grade": models.IntegerField(
            verbose_name="Grade",
            choices=GRADING_SCALE_WITH_NOT_GRADED,
            null=True,
            blank=True,
        ),
        f"{utest_id}_grade_description": models.CharField(
            verbose_name="Grade description",
            max_length=250,
            null=True,
            blank=True,
        ),
    }



from django.conf import settings

from edc_auth.constants import PII, PII_VIEW
from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import consent_codenames, navbar_tuples


def update_site_auths():

    site_auths.add_post_update_func(
        "edc_consent",
        remove_default_model_permissions_from_edc_permissions,
    )

    site_auths.add_custom_permissions_tuples(
        model="edc_consent.edcpermissions", codename_tuples=navbar_tuples
    )

    site_auths.update_group(*consent_codenames, name=PII, no_delete=True)
    site_auths.update_group(*consent_codenames, name=PII_VIEW, view_only=True)
    site_auths.add_pii_model(settings.SUBJECT_CONSENT_MODEL)


update_site_auths()



# Generated by Django 4.2.7 on 2023-12-05 03:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_facility", "0011_remove_healthfacilitytypes_edc_facilit_name_78007a_idx"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="holiday",
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name="holiday",
            constraint=models.UniqueConstraint(
                fields=("country", "local_date"),
                name="edc_facility_holiday_country_uniq",
            ),
        ),
    ]



from edc_list_data.model_mixins import ListModelMixin


class ContainerType(ListModelMixin):

    def save(self, *args, **kwargs):
        if not self.display_name:
            self.display_name = self.name.capitalize()
        super().save(*args, **kwargs)

    class Meta(ListModelMixin.Meta):
        verbose_name = "Container type"
        verbose_name_plural = "Container types"



from django.utils.safestring import mark_safe

CELLS_PER_MILLIMETER_CUBED_DISPLAY = mark_safe("cells/mm<sup>3</sup>")  # nosec B308
IU_LITER_DISPLAY = mark_safe("IU/L")  # nosec B308
MICROMOLES_PER_LITER_DISPLAY = "μmol/L (micromoles/L)"
MICRO_IU_MILLILITER_DISPLAY = mark_safe("μIU/mL")  # nosec B308
MILLIMOLES_PER_LITER_DISPLAY = "mmol/L (millimoles/L)"
MILLI_IU_LITER_DISPLAY = mark_safe("mIU/L")  # nosec B308
MM3_DISPLAY = mark_safe("mm<sup>3</sup>")  # nosec B308
TEN_X_3_PER_LITER_DISPLAY = mark_safe("10<sup>3</sup>/L")  # nosec B308
TEN_X_9_PER_LITER_DISPLAY = mark_safe("10<sup>9</sup>/L")  # nosec B308



# Generated by Django 2.2.3 on 2019-09-01 08:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_facility", "0002_auto_20180102_1158")]

    operations = [
        migrations.AlterField(
            model_name="holiday", name="country", field=models.CharField(max_length=50)
        ),
        migrations.AlterField(
            model_name="holiday", name="name", field=models.CharField(max_length=50)
        ),
    ]



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver

from ..constants import DELETE, INSERT, UPDATE


@receiver(post_save, weak=False, dispatch_uid="export_transaction_history_on_post_save")
def export_transaction_history_on_post_save(
    sender, instance, raw, created, using, update_fields, **kwargs
):
    """Serializes the model instance to export history model
    if manager exists."""
    if not raw:
        try:
            change_type = INSERT if created else UPDATE
            sender.export_history.serialize_to_exported_object_history(
                instance, change_type, using=using
            )
        except AttributeError as e:
            if "export_history" not in str(e):
                raise AttributeError(e)


@receiver(pre_delete, weak=False, dispatch_uid="export_transaction_history_on_pre_delete")
def export_transaction_history_on_pre_delete(sender, instance, using, **kwargs):
    """Serializes the model instance, before deleting, to export
    history model if manager exists."""
    try:
        sender.export_history.serialize_to_exported_object_history(
            instance, DELETE, using=using
        )
    except AttributeError as e:
        if "export_history" not in str(e):
            raise



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
import edc_model.validators.date
import edc_protocol.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_visit_schedule",
            "0019_alter_historicalonschedule_device_created_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalonschedule",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="subjectschedulehistory",
            name="schedule_status",
            field=models.CharField(
                choices=[
                    ("onschedule", "On schedule"),
                    ("offschedule", "Off schedule"),
                ],
                default="",
                max_length=15,
            ),
        ),
    ]



import pandas as pd

from ..constants import timedelta_datatypes


def convert_timedelta_from_model(source_df: pd.DataFrame, model_cls) -> pd.DataFrame:
    date_cols = []
    for field_cls in model_cls._meta.get_fields():
        if field_cls.get_internal_type() in timedelta_datatypes:
            date_cols.append(field_cls.name)
    if date_cols:
        source_df[date_cols] = source_df[date_cols].apply(pd.to_timedelta, errors="coerce")
    return source_df



import re
import secrets

from edc_protocol.research_protocol_config import ResearchProtocolConfig

from .exceptions import SubjectIdentifierError


def is_subject_identifier_or_raise(subject_identifier, reference_obj=None, raise_on_none=None):
    """Returns the given subject identifier.

    * If the format of the `subject_identifier` is invalid,
      raises an exception.
    * If `subject_identifier` is None, does nothing, unless
      `raise_on_none` is `True`.
    """
    valid_subject_identifier = subject_identifier and re.match(
        ResearchProtocolConfig().subject_identifier_pattern, subject_identifier or ""
    )
    if not valid_subject_identifier or (not subject_identifier and raise_on_none):
        reference_msg = ""
        if reference_obj:
            reference_msg = f"See {reference_obj!r}. "
        raise SubjectIdentifierError(
            f"Invalid format for subject identifier. {reference_msg}"
            f"Got `{subject_identifier or ''}`. "
            f"Expected pattern `{ResearchProtocolConfig().subject_identifier_pattern}`"
        )
    return subject_identifier


def get_human_phrase(no_hyphen: bool | None = None) -> str:
    """Returns 6 digits split by a '-', e.g. DEC-96E.

    There are 213,127,200 permutations from an unambiguous alphabet.
    """

    alphabet = "ABCDEFGHKMNPRTUVWXYZ2346789"
    phrase = "".join(secrets.choice(alphabet) for i in range(6)).upper()
    if no_hyphen:
        return phrase
    return phrase[0:3] + "-" + phrase[3:6]


def convert_to_human_readable(identifier: str) -> str:
    if not identifier:
        return ""
    return "-".join(re.findall(r".{1,4}", identifier))



from .grading_data import GradingData
from .grading_exception import GradingException
from .mw import MolecularWeight
from .normal_data import NormalData
from .reference_model_mixins import ReferenceModelMixin
from .reference_range_collection import ReferenceRangeCollection



# Generated by Django 4.1.2 on 2023-01-25 00:06

import django_crypto_fields.fields.lastname_field
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_registration", "0025_auto_20220914_0038"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalregisteredsubject",
            name="familiar_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Familiar name",
            ),
        ),
        migrations.AddField(
            model_name="historicalregisteredsubject",
            name="full_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Full name",
            ),
        ),
        migrations.AddField(
            model_name="registeredsubject",
            name="familiar_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Familiar name",
            ),
        ),
        migrations.AddField(
            model_name="registeredsubject",
            name="full_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Full name",
            ),
        ),
    ]



from __future__ import annotations

from typing import TYPE_CHECKING

from clinicedc_constants import YES
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from edc_appointment.utils import get_next_appointment
from edc_utils.text import formatted_datetime
from edc_visit_tracking.utils import get_next_related_visit

from ..exceptions import NextStudyMedicationError, StudyMedicationError
from ..refill import create_refills_from_crf
from .previous_model_mixin import PreviousNextModelMixin
from .study_medication_refill_model_mixin import StudyMedicationRefillModelMixin

if TYPE_CHECKING:
    from ..models import RxRefill


class StudyMedicationCrfModelMixin(PreviousNextModelMixin, StudyMedicationRefillModelMixin):
    """Declare with field subject_visit using a CRF model mixin"""

    def save(self, *args, **kwargs):
        if not self.formulation:
            raise StudyMedicationError(
                f"Formulation cannot be None. Perhaps catch this in the form. See {self}."
            )
        if not self.dosage_guideline:
            raise StudyMedicationError(
                f"Dosage guideline cannot be None. Perhaps catch this in the form. See {self}."
            )

        if self.refill_to_next_visit == YES and not get_next_appointment(
            self.related_visit.appointment, include_interim=True
        ):
            raise NextStudyMedicationError(
                "Cannot refill to next appointment. This subject has no future appointments. "
                f"Perhaps catch this in the form. See {self.related_visit}."
            )

        if self.refill_to_next_visit == YES:
            # overwrite the value of refill_end_datetime coming from
            # the form with the date of next visit or appointment.
            self.refill_end_datetime = getattr(
                get_next_related_visit(self.related_visit, include_interim=True),
                "report_datetime",
                None,
            ) or getattr(
                get_next_appointment(self.related_visit.appointment, include_interim=True),
                "appt_datetime",
                None,
            )
        if not self.refill_end_datetime:
            # if None, means there is not a next appointment
            self.refill_end_datetime = self.refill_start_datetime

        self.number_of_days = (self.refill_end_datetime - self.refill_start_datetime).days

        if not self.rx:
            dt = formatted_datetime(self.refill_start_datetime)
            if not self.rx_model_cls.objects.filter(
                subject_identifier=self.related_visit.subject_identifier,
            ).exists():
                error_msg = (
                    f"No prescriptions found for this subject. "
                    f"Using subject_identifier=`{self.related_visit.subject_identifier}` "
                    f"Perhaps catch this in the form. See {self}."
                )
            elif not self.rx_model_cls.objects.filter(
                subject_identifier=self.related_visit.subject_identifier,
                medications__in=[self.formulation.medication],
            ).exists():
                error_msg = (
                    f"No prescriptions found for this medication. "
                    f"Using medication {self.formulation.medication}. "
                    f"Perhaps catch this in the form. See {self}."
                )
            else:
                error_msg = (
                    f"A valid prescription not found. Check refill date. "
                    f"Using refill start datetime `{dt}`. "
                    f"Perhaps catch this in the form. See {self}."
                )

            raise StudyMedicationError(error_msg)

        super().save(*args, **kwargs)

    def creates_refills_from_crf(self) -> tuple[RxRefill, RxRefill | None]:
        """Attribute called in signal"""
        return create_refills_from_crf(self, self.related_visit_model_attr())

    def get_subject_identifier(self):
        return self.related_visit.subject_identifier

    @property
    def rx_model_cls(self):
        return django_apps.get_model("edc_pharmacy.rx")

    @property
    def rx(self):
        try:
            rx = self.rx_model_cls.objects.get(
                registered_subject__subject_identifier=self.related_visit.subject_identifier,
                medications__in=[self.formulation.medication],
                rx_date__lte=self.refill_start_datetime.date(),
            )
        except ObjectDoesNotExist:
            return None
        else:
            if (
                rx.rx_expiration_date
                and rx.rx_expiration_date < self.refill_end_datetime.date()
            ):
                raise StudyMedicationError(
                    f"Prescription is expired. Got {rx}. Perhaps catch this in the form. "
                    f"See {self}."
                )
        return rx

    class Meta(StudyMedicationRefillModelMixin.Meta):
        abstract = True



from __future__ import annotations

from django.core.exceptions import ObjectDoesNotExist

from .models import SiteProfile
from .site import SiteNotRegistered, sites


class SiteViewMixin:
    def get_context_data(self, **kwargs) -> dict:
        kwargs = self.get_context_data_for_sites(**kwargs)
        return super().get_context_data(**kwargs)

    def get_context_data_for_sites(self, **kwargs):
        try:
            site_profile = SiteProfile.objects.get(site__id=self.request.site.id)
        except ObjectDoesNotExist:
            site_profile = None
        kwargs.update(site_profile=site_profile)
        try:
            kwargs.update(site_title=site_profile.title)
        except AttributeError as e:
            if not sites.all():
                raise SiteNotRegistered(
                    "Unable to determine site profile 'title'. No sites have been registered! "
                ) from e
            raise
        return kwargs



from uuid import uuid4

from django.core.validators import RegexValidator
from django.db import models
from django_crypto_fields.fields import EncryptedCharField


class NameFieldsModelMixin(models.Model):
    legal_name = EncryptedCharField(
        verbose_name="Full name",
        blank=False,
        validators=[
            RegexValidator(
                regex=r"^(([A-Z]+ )*[A-Z]+)?$",
                message=(
                    "Ensure full name consists of letters only in upper case "
                    "separated by single spaces"
                ),
            )
        ],
    )

    familiar_name = EncryptedCharField(
        verbose_name="By what NAME should we refer to you? (if we speak to you)",
        blank=False,
        help_text=(
            "Should be a name. Do NOT use MR, MRS, MISS, SIR, MADAM and other such titles."
        ),
        validators=[
            RegexValidator(
                regex=r"^(([A-Z]+ )*[A-Z]+)?$",
                message=(
                    "Ensure name consists of letters only in upper case "
                    "separated by single spaces"
                ),
            )
        ],
    )

    initials = EncryptedCharField(
        validators=[
            RegexValidator(
                regex=r"^[A-Z]{2,3}$",
                message="Ensure initials consist of letters only in upper case, no spaces.",
            )
        ],
        null=True,
        blank=False,
    )

    def save(self, *args, **kwargs):
        if not self.familiar_name:
            self.familiar_name = str(uuid4())
        if not self.legal_name:
            self.legal_name = str(uuid4())
        super().save(*args, **kwargs)

    class Meta:
        abstract = True



from __future__ import annotations

from typing import TYPE_CHECKING

from edc_protocol.research_protocol_config import ResearchProtocolConfig
from edc_utils.text import convert_from_camel

from ..url_config import UrlConfig

if TYPE_CHECKING:
    from django.urls import URLPattern


class UrlRequestContextError(Exception):
    pass


class UrlRequestContextMixin:
    urlconfig_getattr = "dashboard_urls"
    urlconfig_identifier_label = "subject_identifier"
    urlconfig_identifier_pattern = ResearchProtocolConfig().subject_identifier_pattern
    urlconfig_label = None
    url_name = None

    # @classmethod
    # def get_urlname(cls):
    #     return cls.url_name

    @classmethod
    def urls(
        cls,
        *,
        namespace: str,
        url_names_key: str,
        identifier_label: str | None = None,
        identifier_pattern: str | None = None,
    ) -> list[URLPattern]:
        """Returns a list of url patterns generated by UrlConfig.

        Each url pattern has the same name; `cls.url_name`.
        """
        url_names_key = (
            url_names_key
            or cls.urlconfig_label
            or convert_from_camel(cls.__name__.replace("view", "")).lower()
        )
        urlconfig = UrlConfig(
            view_class=cls,
            namespace=namespace,
            url_names_key=url_names_key,
            identifier_label=identifier_label or cls.urlconfig_identifier_label,
            identifier_pattern=identifier_pattern or cls.urlconfig_identifier_pattern,
        )
        if cls.urlconfig_getattr not in [
            "dashboard_urls",
            "listboard_urls",
            "review_listboard_urls",
        ]:
            raise UrlRequestContextError(
                f"Invalid urlconfig attr. Got {cls.urlconfig_getattr}."
            )
        return getattr(urlconfig, cls.urlconfig_getattr)

    # @staticmethod
    # def add_url_to_context(new_key=None, existing_key=None) -> dict[str, str]:
    #     """Add url as new_key to the context using the value
    #     of the existing_key from request.context_data.
    #     """
    #     if new_key != existing_key:
    #         try:
    #             url_data = {new_key: url_names.get(existing_key)}
    #         except InvalidDashboardUrlName as e:
    #             raise UrlRequestContextError(
    #                 f"Url name not defined in url_names. "
    #                 f"Expected one of {url_names.registry}. Got {e}. "
    #                 f"Hint: check if dashboard middleware is loaded."
    #             ) from e
    #         return url_data
    #     return {existing_key: url_names.get(existing_key)}



from typing import Any

from django.views.generic.base import TemplateView

from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from .research_protocol_config import ResearchProtocolConfig


class HomeView(EdcViewMixin, NavbarViewMixin, TemplateView):
    template_name = "edc_protocol/home.html"
    navbar_name = "edc_protocol"
    navbar_selected_item = "protocol"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        protocol_config = ResearchProtocolConfig()
        kwargs.update(
            {
                "protocol": protocol_config.protocol,
                "protocol_number": protocol_config.protocol_number,
                "protocol_name": protocol_config.protocol_name,
                "protocol_title": protocol_config.protocol_title,
                "study_open_datetime": protocol_config.study_open_datetime,
                "study_close_datetime": protocol_config.study_close_datetime,
            }
        )
        return super().get_context_data(**kwargs)



from celery.result import AsyncResult
from django.http import JsonResponse
from django.views import View


class CeleryTaskStatusView(View):
    def get(self, request, *args, **kwargs):  # noqa: ARG002
        task_id = request.GET.get("task_id")
        response_data = {"task_id": task_id, "status": None}
        try:
            result = AsyncResult(str(task_id or ""))
        except (TypeError, ValueError):
            result = None
        if getattr(result, "id", None):
            response_data.update({"status": result.status})
        return JsonResponse(response_data)



from edc_auth.constants import (
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import (
    ACTION_ITEM,
    ACTION_ITEM_EXPORT,
    ACTION_ITEM_VIEW_ONLY,
    action_items_codenames,
    navbar_tuples,
)


def update_site_auths():

    site_auths.add_post_update_func(
        "edc_action_item",
        remove_default_model_permissions_from_edc_permissions,
    )

    site_auths.add_custom_permissions_tuples(
        model="edc_action_item.edcpermissions", codename_tuples=navbar_tuples
    )

    site_auths.add_group(*action_items_codenames, name=ACTION_ITEM)
    site_auths.add_group(*action_items_codenames, name=ACTION_ITEM_VIEW_ONLY, view_only=True)
    site_auths.add_group(
        "edc_action_item.export_actionitem",
        "edc_action_item.export_actiontype",
        "edc_action_item.export_historicalactionitem",
        name=ACTION_ITEM_EXPORT,
    )

    site_auths.add_custom_permissions_tuples(
        model="edc_action_item.historicalactionitem",
        codename_tuples=[
            (
                "edc_action_item.export_historicalactionitem",
                "Cane export historicalactionitem",
            ),
            (
                "edc_action_item.export_historicalreference",
                "Cane export historicalreference",
            ),
        ],
    )

    site_auths.update_role(ACTION_ITEM, name=CLINICIAN_ROLE)
    site_auths.update_role(ACTION_ITEM, name=NURSE_ROLE)
    site_auths.update_role(ACTION_ITEM, name=CLINICIAN_SUPER_ROLE)
    site_auths.update_role(ACTION_ITEM_VIEW_ONLY, name=AUDITOR_ROLE)

update_site_auths()



from django.db import models


class InlineVisitMethodsModelMixin(models.Model):
    @property
    def visit_code(self):
        return self.subject_visit.visit_code

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier

    class Meta:
        abstract = True



import csv
from pathlib import Path

from django.conf import settings
from django.core.checks import Error, Warning
from django.core.management import color_style

from edc_sites.site import sites as site_sites

style = color_style()


def holiday_path_check(app_configs, holiday_path: str = None, **kwargs):
    errors = []
    if not getattr(settings, "HOLIDAY_FILE", None):
        errors.append(
            Error(
                "Holiday file path not set! See settings.HOLIDAY_FILE.\n",
                id="edc_facility.E001",
            )
        )
    else:
        holiday_path = Path(holiday_path or settings.HOLIDAY_FILE).expanduser()
        if not holiday_path.exists():
            errors.append(
                Warning(
                    f"Holiday file not found! settings.HOLIDAY_FILE={holiday_path}. \n",
                    id="edc_facility.W001",
                )
            )
    return errors


def holiday_country_check(app_configs, holiday_path: str = None, **kwargs):
    errors = []
    holiday_path = Path(holiday_path or settings.HOLIDAY_FILE).expanduser()
    if site_sites.all():
        with holiday_path.open(mode="r") as f:
            reader = csv.DictReader(f, fieldnames=["local_date", "label", "country"])
            next(reader, None)
            for row in reader:
                if row["country"] not in site_sites.countries:
                    errors.append(
                        Warning(
                            "Holiday file has no records for country! Sites are registered "
                            f"for these countries: `{'`, `'.join(site_sites.countries)}`. Got "
                            f"`{row['country']}`\n",
                            id="edc_facility.W002",
                        )
                    )
                    break
    return errors



from __future__ import annotations

from clinicedc_constants import YES


class FastingFormValidatorMixin:
    def validate_fasting_required_fields(self, fasting_prefix: str | None = None):
        """Uses fields `fasting`,`fasting_duration_str`"""
        fasting_prefix = fasting_prefix or "fasting"
        self.required_if(
            YES, field=fasting_prefix, field_required=f"{fasting_prefix}_duration_str"
        )



from __future__ import annotations

import contextlib

from clinicedc_constants import CLOSED, NEW
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from tqdm import tqdm

from .create_or_update_action_type import create_or_update_action_type
from .identifiers import ActionIdentifier
from .models import ActionItem
from .site_action_items import AlreadyRegistered, site_action_items


def update_action_identifier(model=None, action_cls=None, apps=None, status=None):
    """Update an action_identifier field for an existing model
    recently updated to use an action class.

    For example, in a migration (RunPython):
        def update_followup_examination_action_identifier(apps, schema_editor):
            update_action_identifier(
                    model="meta_subject.followupexamination",
                    action_cls=FollowupExaminationAction,
                    apps=apps,
            )
    """

    apps = apps or django_apps
    action_item_cls = apps.get_model("edc_action_item.actionitem")
    model_cls = apps.get_model(model)
    action_type = create_or_update_action_type(apps=apps, **action_cls.as_dict())
    for obj in tqdm(model_cls.objects.filter(action_identifier__isnull=True)):
        action_item = action_item_cls(
            subject_identifier=obj.related_visit.subject_identifier,
            action_type=action_type,
            action_identifier=ActionIdentifier().identifier,
            site=obj.related_visit.site,
        )
        action_item.linked_to_reference = True
        action_item.status = status or CLOSED
        action_item.save()
        obj.action_identifier = action_item.action_identifier
        obj.action_item = action_item
        obj.save_base(update_fields=["action_identifier", "action_item"])


def reset_and_delete_action_item(instance, using=None):
    """Called by signal"""
    action_item = ActionItem.objects.using(using).get(
        action_identifier=instance.action_identifier
    )
    action_item.status = NEW
    action_item.linked_to_reference = False
    action_item.save(using=using)
    for obj in ActionItem.objects.using(using).filter(
        parent_action_item=action_item,
        status=NEW,
    ):
        obj.delete(using=using)
    for obj in ActionItem.objects.using(using).filter(
        related_action_item=action_item,
        status=NEW,
    ):
        obj.delete(using=using)
    if action_item.action.delete_with_reference_object:
        action_item.delete()


def register_actions(*action_cls):
    for cls in action_cls:
        with contextlib.suppress(AlreadyRegistered):
            site_action_items.register(cls)


def get_reference_obj(action_item: ActionItem | None):
    """Returns the reference model instance or None."""
    try:
        reference_obj = action_item.reference_obj
    except (AttributeError, ObjectDoesNotExist):
        reference_obj = None
    return reference_obj


def get_parent_reference_obj(action_item: ActionItem | None):
    """Returns the parent reference model instance or None."""
    try:
        reference_obj = action_item.parent_action_item.reference_obj
    except (AttributeError, ObjectDoesNotExist):
        reference_obj = None
    return reference_obj


def get_related_reference_obj(action_item: ActionItem | None):
    """Returns the change url for the related reference
    model instance or None.
    """
    try:
        reference_obj = action_item.related_action_item.reference_obj
    except (AttributeError, ObjectDoesNotExist):
        reference_obj = None
    return reference_obj



CHART = "chart"
COMPLETED_PROTOCOL_VISIT = "completed protocol"
DEFERRED_VISIT = "deferred"
LOST_VISIT = "lost"
MISSED_VISIT = "missed"
SCHEDULED = "scheduled"
UNSCHEDULED = "unscheduled"
NO_FOLLOW_UP_REASONS = [
    LOST_VISIT,
    DEFERRED_VISIT,
    COMPLETED_PROTOCOL_VISIT,
]
FOLLOW_UP_REASONS = [SCHEDULED, UNSCHEDULED, MISSED_VISIT]
REQUIRED_REASONS = NO_FOLLOW_UP_REASONS + FOLLOW_UP_REASONS
VISIT_MISSED_ACTION = "submit_visit_missed"



# Generated by Django 4.2.7 on 2023-12-03 15:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_metadata",
            "0026_rename_model_verbose_name_crfmetadata_document_name_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="crfmetadata",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Crf collection status",
                "verbose_name_plural": "Crf collection status",
            },
        ),
        migrations.AlterModelOptions(
            name="requisitionmetadata",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Requisition collection status",
                "verbose_name_plural": "Requisition collection status",
            },
        ),
        migrations.RemoveIndex(
            model_name="crfmetadata",
            name="edc_metadat_subject_1b2c7e_idx",
        ),
        migrations.RemoveIndex(
            model_name="requisitionmetadata",
            name="edc_metadat_subject_fb1a00_idx",
        ),
        migrations.AlterUniqueTogether(
            name="crfmetadata",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="crfmetadata",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="crfmetadata",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="requisitionmetadata",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="requisitionmetadata",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="crfmetadata",
            name="entry_status",
            field=models.CharField(
                choices=[
                    ("REQUIRED", "New"),
                    ("KEYED", "Keyed"),
                    ("missed", "Missed"),
                    ("NOT_REQUIRED", "Not required"),
                ],
                default="REQUIRED",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="requisitionmetadata",
            name="entry_status",
            field=models.CharField(
                choices=[
                    ("REQUIRED", "New"),
                    ("KEYED", "Keyed"),
                    ("missed", "Missed"),
                    ("NOT_REQUIRED", "Not required"),
                ],
                default="REQUIRED",
                max_length=25,
            ),
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(
                fields=[
                    "site",
                    "entry_status",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "subject_identifier",
                    "schedule_name",
                    "visit_schedule_name",
                ],
                name="edc_metadata_crfm_site_ent_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "entry_status",
                    "timepoint",
                    "show_order",
                ],
                name="edc_metadata_crfm_subjid_v_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=[
                    "site",
                    "entry_status",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "panel_name",
                    "subject_identifier",
                    "schedule_name",
                    "visit_schedule_name",
                ],
                name="edc_metadat_site_id_cc31cf_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "panel_name",
                    "entry_status",
                    "show_order",
                ],
                name="edc_metadat_subject_51288a_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="requisitionmetadata",
            index=models.Index(
                fields=[
                    "entry_status",
                    "model",
                    "panel_name",
                    "schedule_name",
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                    "visit_schedule_name",
                ],
                name="edc_metadat_entry_s_865e45_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="crfmetadata",
            constraint=models.UniqueConstraint(
                fields=(
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                ),
                name="edc_metadata_crfmetadata_subject_iden_visit_uniq",
            ),
        ),
    ]



from .model_mixins import EdcPermissionsModelMixin


def is_custom_permissions_model(model=None) -> bool:
    """Returns True if model is a permissions model."""
    return issubclass(model, (EdcPermissionsModelMixin,))



def convert_visit_code_to_float(df):
    """Convert visit_code to float using visit_code_sequence"""
    df["visit_code"] = df["visit_code"].astype(float)
    df["visit_code_sequence"] = df["visit_code_sequence"].astype(float)
    df["visit_code_sequence"] = df["visit_code_sequence"].apply(
        lambda x: x / 10.0 if x > 0.0 else 0.0
    )
    df["visit_code"] = df["visit_code"] + df["visit_code_sequence"]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-05 19:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_lab", "0005_auto_20170305_2109")]

    operations = [
        migrations.AddField(
            model_name="aliquot",
            name="shipped",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="historicalaliquot",
            name="shipped",
            field=models.BooleanField(default=False),
        ),
    ]



import pandas as pd
from clinicedc_constants import FEMALE, MALE

from ..constants import (
    MEDIAN_IQR,
    MEDIAN_RANGE,
    N_ONLY,
    N_WITH_COL_PROP,
    N_WITH_ROW_PROP,
)
from ..row import RowDefinition, RowDefinitions
from ..table import Table


class WaistCircumferenceTable(Table):

    def __init__(self, main_df: pd.DataFrame = None):
        super().__init__(
            colname="waist_circumference",
            main_df=main_df,
            title="Waist circumference (cm)",
        )

    @property
    def row_definitions(self) -> RowDefinitions:
        df_tmp = self.main_df.copy()
        row_defs = RowDefinitions(reverse_rows=False)
        row0 = RowDefinition(
            title=self.title,
            label=self.default_sublabel,
            condition=(df_tmp["gender"].notna()),
            columns={FEMALE: (N_ONLY, 2), MALE: (N_ONLY, 2), "All": (N_ONLY, 2)},
            drop=False,
        )
        row_defs.add(row0)

        columns = {
            FEMALE: (N_WITH_COL_PROP, 2),
            MALE: (N_WITH_COL_PROP, 2),
            "All": (N_WITH_ROW_PROP, 2),
        }

        cond_lt_102 = (
            (self.main_df[self.colname] < 102.0) & (self.main_df["gender"] == "Male")
        ) | ((self.main_df[self.colname] < 88.0) & (self.main_df["gender"] == "Female"))
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="Women<88 / Men<102",
                condition=cond_lt_102,
                columns=columns,
                drop=False,
            )
        )
        cond_gte_102 = (
            (self.main_df[self.colname] >= 102.0) & (self.main_df["gender"] == "Male")
        ) | ((self.main_df[self.colname] >= 88.0) & (self.main_df["gender"] == "Female"))
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="Women>=88 / Men>=102",
                condition=cond_gte_102,
                columns=columns,
                drop=False,
            )
        )
        cond_gte_missing = self.main_df[self.colname].isna()
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="not measured",
                condition=cond_gte_missing,
                columns=columns,
                drop=False,
            )
        )

        columns = {
            FEMALE: (MEDIAN_RANGE, 2),
            MALE: (MEDIAN_RANGE, 2),
            "All": (MEDIAN_RANGE, 2),
        }
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="Median (range)",
                condition=(self.main_df[self.colname].notna()),
                columns=columns,
            )
        )

        columns = {
            FEMALE: (MEDIAN_IQR, 2),
            MALE: (MEDIAN_IQR, 2),
            "All": (MEDIAN_IQR, 2),
        }
        row_defs.add(
            RowDefinition(
                colname=self.colname,
                label="Median (IQR)",
                condition=(self.main_df[self.colname].notna()),
                columns=columns,
            )
        )
        return row_defs



from .address_model_admin_mixin import AddressModelAdminMixin
from .inlines import (
    InlineHideOriginalObjectNameMixin,
    LimitedAdminInlineMixin,
    StackedInlineModelAdminMixin,
    TabularInlineMixin,
)
from .model_admin_bypass_default_form_cls_mixin import (
    ModelAdminBypassDefaultFormClsMixin,
)
from .model_admin_form_auto_number_mixin import ModelAdminFormAutoNumberMixin
from .model_admin_form_instructions_mixin import ModelAdminFormInstructionsMixin
from .model_admin_hide_delete_button_on_condition import (
    ModelAdminHideDeleteButtonOnCondition,
)
from .model_admin_institution_mixin import ModelAdminInstitutionMixin
from .model_admin_limit_to_selected_foreignkey import (
    ModelAdminLimitToSelectedForeignkey,
)
from .model_admin_model_redirect_mixin import ModelAdminModelRedirectMixin
from .model_admin_next_url_redirect_mixin import (
    ModelAdminNextUrlRedirectError,
    ModelAdminNextUrlRedirectMixin,
)
from .model_admin_protect_pii_mixin import ModelAdminProtectPiiMixin
from .model_admin_redirect_all_to_changelist_mixin import (
    ModelAdminRedirectAllToChangelistMixin,
)
from .model_admin_redirect_on_delete_mixin import ModelAdminRedirectOnDeleteMixin
from .model_admin_replace_label_text_mixin import ModelAdminReplaceLabelTextMixin
from .templates_model_admin_mixin import TemplatesModelAdminMixin



# Generated by Django 3.2.6 on 2021-09-08 20:00

import uuid

import _socket
import django.contrib.auth.models
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import simple_history.models
from django.conf import settings
from django.db import migrations, models

import edc_identifier.managers
import edc_sites.models
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("edc_action_item", "0028_auto_20210203_0706"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="UnblindingRequestorUser",
            fields=[],
            options={
                "verbose_name": "Unblinding Requestor (User)",
                "verbose_name_plural": "Unblinding Requestors (Users)",
                "proxy": True,
                "default_permissions": ("view",),
                "indexes": [],
                "constraints": [],
            },
            bases=("auth.user",),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="UnblindingReviewerUser",
            fields=[],
            options={
                "verbose_name": "Unblinding Reviewer (User)",
                "verbose_name_plural": "Unblinding Reviewers (Users)",
                "proxy": True,
                "default_permissions": ("view",),
                "indexes": [],
                "constraints": [],
            },
            bases=("auth.user",),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="UnblindingReview",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(max_length=30, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "approved",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("tbd", "To be determined"),
                        ],
                        default="tbd",
                        max_length=15,
                    ),
                ),
                ("comment", models.TextField(null=True, verbose_name="Comment")),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "reviewer",
                    models.ForeignKey(
                        help_text="Select a name from the list",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_unblinding.unblindingrevieweruser",
                        verbose_name="Unblinding request reviewed by",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Unblinding Review",
                "verbose_name_plural": "Unblinding Reviews",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.CreateModel(
            name="UnblindingRequest",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(max_length=30, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "initials",
                    models.CharField(
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters.",
                        max_length=3,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                        verbose_name="Subject's initials",
                    ),
                ),
                (
                    "unblinding_reason",
                    models.TextField(verbose_name="Reason for unblinding"),
                ),
                (
                    "approved",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("tbd", "To be determined"),
                        ],
                        default="tbd",
                        max_length=15,
                    ),
                ),
                ("approved_datetime", models.DateTimeField(null=True)),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "requestor",
                    models.ForeignKey(
                        help_text="Select a name from the list",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_unblinding.unblindingrequestoruser",
                        verbose_name="Unblinding requested by",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Unblinding Request",
                "verbose_name_plural": "Unblinding Requests",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalUnblindingReview",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(db_index=True, max_length=30)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "approved",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("tbd", "To be determined"),
                        ],
                        default="tbd",
                        max_length=15,
                    ),
                ),
                ("comment", models.TextField(null=True, verbose_name="Comment")),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "reviewer",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        help_text="Select a name from the list",
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_unblinding.unblindingrevieweruser",
                        verbose_name="Unblinding request reviewed by",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Unblinding Review",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalUnblindingRequest",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(db_index=True, max_length=30)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "initials",
                    models.CharField(
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters.",
                        max_length=3,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                        verbose_name="Subject's initials",
                    ),
                ),
                (
                    "unblinding_reason",
                    models.TextField(verbose_name="Reason for unblinding"),
                ),
                (
                    "approved",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("tbd", "To be determined"),
                        ],
                        default="tbd",
                        max_length=15,
                    ),
                ),
                ("approved_datetime", models.DateTimeField(null=True)),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "requestor",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        help_text="Select a name from the list",
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_unblinding.unblindingrequestoruser",
                        verbose_name="Unblinding requested by",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Unblinding Request",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.AddIndex(
            model_name="unblindingreview",
            index=models.Index(
                fields=["subject_identifier", "action_identifier", "site", "id"],
                name="edc_unblind_subject_e0282c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="unblindingrequest",
            index=models.Index(
                fields=["subject_identifier", "action_identifier", "site", "id"],
                name="edc_unblind_subject_f9ae72_idx",
            ),
        ),
    ]



from decimal import Decimal

from celery.states import PENDING
from django.contrib import admin
from django.contrib.admin.widgets import AutocompleteSelect
from django.template.loader import render_to_string
from django.urls import reverse
from django_audit_fields import audit_fieldset_tuple
from rangefilter.filters import DateRangeFilterBuilder

from edc_model_admin.history import SimpleHistoryAdmin
from edc_utils.celery import get_task_result
from edc_utils.date import to_local

from ...admin_site import edc_pharmacy_admin
from ...forms import RepackRequestForm
from ...models import RepackRequest
from ...utils import format_qty
from ..actions import (
    confirm_repacked_stock_action,
    print_labels_from_repack_request,
    process_repack_request_action,
)
from ..list_filters import AssignmentListFilter as BaseAssignmentListFilter
from ..model_admin_mixin import ModelAdminMixin
from ..remove_fields_for_blinded_users import remove_fields_for_blinded_users


class AssignmentListFilter(BaseAssignmentListFilter):
    title = "Assignment"
    parameter_name = "assignment"
    lookup_str = "from_stock__product__assignment__name"


@admin.register(RepackRequest, site=edc_pharmacy_admin)
class RequestRepackAdmin(ModelAdminMixin, SimpleHistoryAdmin):
    change_list_title = "Pharmacy: Repack/Decant request"
    change_form_title = "Pharmacy: Repack/Decant"
    history_list_display = ()

    show_object_tools = True
    show_cancel = True
    list_per_page = 20

    ordering = ("-repack_identifier",)

    autocomplete_fields = ("from_stock", "container")
    form = RepackRequestForm
    actions = (
        process_repack_request_action,
        print_labels_from_repack_request,
        confirm_repacked_stock_action,
    )

    change_list_note = render_to_string(
        "edc_pharmacy/stock/instructions/repack_instructions.html"
    )

    fieldsets = (
        (
            "Section A: Repack",
            {
                "fields": (
                    "repack_identifier",
                    "repack_datetime",
                    "from_stock",
                    "container",
                    "requested_qty",
                    "processed_qty",
                )
            },
        ),
        (
            "Task",
            {"classes": ("collapse",), "fields": ("task_id",)},
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "identifier",
        "repack_date",
        "from_stock_changelist",
        "assignment",
        "stock_changelist",
        "formatted_requested_qty",
        "formatted_processed_qty",
        "confirmed_qty",
        "container",
        "from_stock__product__name",
        "task_status",
    )

    list_filter = (
        ("repack_datetime", DateRangeFilterBuilder()),
        AssignmentListFilter,
    )

    search_fields = (
        "id",
        "container__name",
        "from_stock__code",
    )

    readonly_fields = ("processed_qty", "task_id")

    def get_list_display(self, request):
        fields = super().get_list_display(request)
        return remove_fields_for_blinded_users(request, fields)

    def get_list_filter(self, request):
        fields = super().get_list_filter(request)
        return remove_fields_for_blinded_users(request, fields)

    def get_search_fields(self, request):
        fields = super().get_search_fields(request)
        return remove_fields_for_blinded_users(request, fields)

    @admin.display(description="Repack date", ordering="repack_datetime")
    def repack_date(self, obj):
        return to_local(obj.repack_datetime).date()

    @admin.display(
        description="Assignment",
        ordering="from_stock__product__assignment__display_name",
    )
    def assignment(self, obj):
        return obj.from_stock.product.assignment.display_name

    @admin.display(description="Stock")
    def stock_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_stock_changelist")
        url = f"{url}?q={obj.id}"
        context = dict(url=url, label="Stock", title="Go to stock")
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="From stock", ordering="from_stock__code")
    def from_stock_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_stock_changelist")
        url = f"{url}?q={obj.from_stock.code}&decanted=No"
        context = dict(url=url, label=obj.from_stock.code, title="Go to stock")
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="REPACK #", ordering="-repack_identifier")
    def identifier(self, obj):
        return obj.repack_identifier

    @admin.display(description="Requested", ordering="requested_qty")
    def formatted_requested_qty(self, obj):
        return format_qty(obj.requested_qty, obj.container)

    @admin.display(description="Created", ordering="processed_qty")
    def formatted_processed_qty(self, obj):
        result = get_task_result(obj)
        if getattr(result, "status", "") == PENDING:
            return PENDING
        return format_qty(obj.processed_qty, obj.container)

    @admin.display(description="Confirmed")
    def confirmed_qty(self, obj):
        return obj.stock_set.filter(confirmation__isnull=False).count()

    @admin.display(description="Task")
    def task_status(self, obj):
        result = get_task_result(obj)
        return getattr(result, "status", None)

    def get_readonly_fields(self, request, obj=None):  # noqa: ARG002
        if obj and (obj.processed_qty or Decimal(0)) > Decimal(0):
            f = [
                "repack_identifier",
                "repack_datetime",
                "container",
                "from_stock",
            ]
            return self.readonly_fields + tuple(f)
        return self.readonly_fields

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "from_stock" and request.GET.get("from_stock"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("from_stock", 0)
            )
            kwargs["widget"] = AutocompleteSelect(
                db_field.remote_field, self.admin_site, using=kwargs.get("using")
            )

        return super().formfield_for_foreignkey(db_field, request, **kwargs)



from uuid import uuid4

from django.db import models
from django.utils import timezone

from edc_model.models import BaseUuidModel

from ..stock import Location


class Room(BaseUuidModel):
    room_identifier = models.CharField(max_length=36, default=uuid4, unique=True)

    room_datetime = models.DateTimeField(default=timezone.now)

    name = models.CharField(max_length=25, unique=True)

    description = models.TextField(default="")

    location = models.ForeignKey(Location, on_delete=models.PROTECT)

    def __str__(self):
        return f"Room {self.name}"

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Room"
        verbose_name_plural = "Rooms"



from ..constants import (
    ACCOUNT_MANAGER,
    ADMINISTRATION,
    AUDITOR,
    CELERY_MANAGER,
    CLINIC,
    CLINIC_SUPER,
    EVERYONE,
    PII,
    PII_VIEW,
)
from .codenames import account_manager, administration, celery_manager, everyone


def get_default_groups() -> dict[str, list[str]]:
    return {
        ACCOUNT_MANAGER: account_manager,
        ADMINISTRATION: administration,
        AUDITOR: [],
        CELERY_MANAGER: celery_manager,
        CLINIC: [],
        CLINIC_SUPER: [],
        EVERYONE: everyone,
        PII: [],
        PII_VIEW: [],
    }



# Generated by Django 2.2.6 on 2019-10-26 19:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_auth", "0016_auto_20191026_2153")]

    operations = [
        migrations.AlterField(
            model_name="role",
            name="name",
            field=models.CharField(
                db_index=True,
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        )
    ]



from __future__ import annotations

from typing import TYPE_CHECKING, Any
from warnings import warn

from django.apps import apps as django_apps
from django.contrib.admin.sites import all_sites
from django.core.exceptions import MultipleObjectsReturned
from django.db.models import QuerySet

from .metadata import model_cls_registered_with_admin_site

if TYPE_CHECKING:
    from edc_appointment.models import Appointment
    from edc_visit_tracking.typing_stubs import RelatedVisitProtocol

    from ..models import CrfMetadata, RequisitionMetadata


class MetadataGetterError(Exception):
    pass


class MetadataValidator:
    def __init__(
        self,
        metadata_obj: CrfMetadata | RequisitionMetadata,
        related_visit: RelatedVisitProtocol,
    ) -> None:
        self.metadata_obj = metadata_obj
        self.related_visit = related_visit
        self.validate_metadata_object()

    @property
    def extra_query_attrs(self) -> dict:
        return {}

    def validate_metadata_object(self) -> None:
        if self.metadata_obj:
            # confirm model class exists
            try:
                source_model_cls = django_apps.get_model(self.metadata_obj.model)
            except LookupError:
                self.metadata_obj.delete()
                self.metadata_obj = None
            else:
                if not model_cls_registered_with_admin_site(source_model_cls):
                    warn(
                        "Model class not registered with Admin. "
                        f"Deleting related metadata. Got {source_model_cls}.",
                        stacklevel=2,
                    )
                    self.metadata_obj.delete()
                    self.metadata_obj = None
                else:
                    # confirm metadata.entry_status is correct
                    query_attrs = {
                        f"{source_model_cls.related_visit_model_attr()}": self.related_visit
                    }
                    query_attrs.update(**self.extra_query_attrs)
                    if source_model_cls.objects.filter(**query_attrs).values("id").count() > 1:
                        raise MultipleObjectsReturned(
                            f"{source_model_cls._meta.label_lower} {self.related_visit}"
                        )
                    # try:
                    #     model_cls.objects.get(**query_attrs)
                    # except AttributeError as e:
                    #     if "related_visit_model_attr" not in str(e):
                    #         raise ImproperlyConfigured(f"{e} See {repr(model_cls)}")
                    #     raise
                    # except ObjectDoesNotExist:
                    #     pass
                    # except MultipleObjectsReturned:
                    #     raise

    @staticmethod
    def model_cls_registered_with_admin_site(model_cls: Any) -> bool:
        """Returns True if model cls is registered in Admin."""
        return any(model_cls in admin_site._registry for admin_site in all_sites)


class MetadataGetter:
    """A class that gets a filtered queryset of metadata --
    `metadata_objects`.

    * gets a queryset of CrfMetadata/RequisitionMetadata instances for
      the given appointment;
    * validates the entry status of each using the
      `metadata_validator_cls` and;
    * returns a requeried queryset.
    """

    metadata_model: str = None

    metadata_validator_cls = MetadataValidator

    def __init__(self, appointment: Appointment) -> None:
        self.options = {}
        self.appointment = appointment
        self.related_visit: RelatedVisitProtocol | None = getattr(
            self.appointment, "related_visit", None
        )
        instance = self.related_visit or self.appointment
        self.subject_identifier = instance.subject_identifier
        self.visit_code = instance.visit_code
        self.visit_code_sequence = instance.visit_code_sequence
        query_options = dict(
            subject_identifier=self.subject_identifier,
            visit_code=self.visit_code,
            visit_code_sequence=self.visit_code_sequence,
            visit_schedule_name=instance.visit_schedule_name,
            schedule_name=instance.schedule_name,
        )
        queryset = self.metadata_model_cls.objects.filter(**query_options).order_by(
            "show_order"
        )
        self.metadata_objects = self.validate_metadata_objects(queryset)

    @property
    def metadata_model_cls(self) -> CrfMetadata | RequisitionMetadata:
        return django_apps.get_model(self.metadata_model)

    def next_object(
        self, show_order: int | None = None, entry_status: str | None = None
    ) -> CrfMetadata | RequisitionMetadata:
        """Returns the next model instance based on the show order."""
        if show_order is None:
            metadata_obj = None
        else:
            opts = {"show_order__gt": show_order}
            if entry_status:
                opts.update(entry_status=entry_status)
            metadata_obj = self.metadata_objects.filter(**opts).order_by("show_order").first()
        return metadata_obj

    def validate_metadata_objects(
        self, queryset: QuerySet[CrfMetadata | RequisitionMetadata]
    ) -> QuerySet[CrfMetadata | RequisitionMetadata]:
        metadata_obj: CrfMetadata | RequisitionMetadata
        for metadata_obj in queryset:
            self.metadata_validator_cls(metadata_obj, self.related_visit)
        return queryset.all()



from django.conf import settings
from django.core.checks import Error


def edc_middleware_check(
    app_configs, app_label=None, middleware_name=None, error_code=None, **kwargs
):
    errors = []
    if middleware_name not in settings.MIDDLEWARE:
        errors.append(
            Error(
                "Missing MIDDLEWARE. " f"Expected `{middleware_name}`.",
                id=f"{app_label}.{error_code or '001'}",
            )
        )
    return errors



# Generated by Django 4.2.7 on 2023-11-28 03:47

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_metadata", "0025_crfmetadata_document_user_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="crfmetadata",
            old_name="model_verbose_name",
            new_name="document_name",
        ),
        migrations.RenameField(
            model_name="requisitionmetadata",
            old_name="model_verbose_name",
            new_name="document_name",
        ),
    ]



from __future__ import annotations

import contextlib
from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.db.models import F

if TYPE_CHECKING:
    from edc_pharmacy.models import Stock


def confirmed_at_current_location(code) -> Stock | None:
    """Returns the stock instance if the location is current.

    To truly be the current location, the stock item must be
    confirmed at the location.
    """
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    with contextlib.suppress(stock_model_cls.DoesNotExist):
        return stock_model_cls.objects.get(
            code=code,
            container__may_dispense_as=True,
            confirmed__isnull=False,
            allocation__isnull=False,
            stocktransferitem__isnull=False,
            location=F("stocktransferitem__stock_transfer__to_location"),
            stocktransferitem__confirmationatlocationitem__isnull=False,
        )
    return None


def is_in_transit(code):
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    return stock_model_cls.objects.filter(
        code=code,
        container__may_dispense_as=True,
        confirmed__isnull=False,
        allocation__isnull=False,
        stocktransferitem__isnull=False,
        location=F("stocktransferitem__stock_transfer__to_location"),
        stocktransferitem__confirmationatlocationitem__isnull=True,
        dispenseitem__isnull=True,
    ).exists()


def is_at_location(code):
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    return stock_model_cls.objects.filter(
        code=code,
        container__may_dispense_as=True,
        confirmed__isnull=False,
        allocation__isnull=False,
        stocktransferitem__isnull=False,
        location=F("stocktransferitem__stock_transfer__to_location"),
        stocktransferitem__confirmationatlocationitem__isnull=False,
        dispenseitem__isnull=True,
    ).exists()


def is_stored_at_location(code):
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    return stock_model_cls.objects.filter(
        code=code,
        container__may_dispense_as=True,
        confirmed__isnull=False,
        allocation__isnull=False,
        stocktransferitem__isnull=False,
        location=F("stocktransferitem__stock_transfer__to_location"),
        stocktransferitem__confirmationatlocationitem__isnull=False,
        storagebinitem__isnull=False,
        dispenseitem__isnull=True,
    ).exists()


def is_dispensed(code):
    stock_model_cls: type[Stock] = django_apps.get_model("edc_pharmacy.stock")
    return stock_model_cls.objects.filter(
        code=code,
        container__may_dispense_as=True,
        confirmed__isnull=False,
        allocation__isnull=False,
        stocktransferitem__isnull=False,
        location=F("stocktransferitem__stock_transfer__to_location"),
        stocktransferitem__confirmationatlocationitem__isnull=False,
        # storagebinitem__isnull=False,
        dispenseitem__isnull=False,
    ).exists()



from .offstudy_crf_model_mixin import OffstudyCrfModelMixin
from .offstudy_model_mixin import OffstudyModelMixin, OffstudyModelMixinError
from .offstudy_non_crf_model_mixin import OffstudyNonCrfModelMixin



from edc_navbar import Navbar, NavbarItem, site_navbars

navbar = Navbar(name="edc_dashboard")
navbar.register(
    NavbarItem(
        name="edc_dashboard",
        label="Edc Dashboard",
        fa_icon="fa-cogs",
        codename="edc_navbar.nav_edc_dashboard",
        url_with_namespace="edc_dashboard:home_url",
    )
)


site_navbars.register(navbar)



from __future__ import annotations

from datetime import date, datetime
from typing import TYPE_CHECKING

from .get_normal_data_or_raise import get_normal_data_or_raise

if TYPE_CHECKING:
    from ..models import ReferenceRangeCollection


__all__ = ["in_normal_bounds_or_raise"]


def in_normal_bounds_or_raise(
    reference_range_collection: ReferenceRangeCollection = None,
    label: str | None = None,
    value: int | float | None = None,
    units: str | None = None,
    gender: str | None = None,
    dob: date | None = None,
    report_datetime: datetime | None = None,
    age_units: str | None = None,
    create_missing_normal: bool | None = None,
) -> bool:
    """Is this used??"""
    obj = get_normal_data_or_raise(
        reference_range_collection=reference_range_collection,
        label=label,
        units=units,
        gender=gender,
        dob=dob,
        report_datetime=report_datetime,
        age_units=age_units,
        create_missing_normal=create_missing_normal,
    )
    return obj.value_in_normal_range_or_raise(
        value=value, dob=dob, report_datetime=report_datetime, age_units=age_units
    )



from django.db import models
from django.db.models import UniqueConstraint
from django.db.models.deletion import PROTECT

from edc_model.models import BaseUuidModel
from edc_search.model_mixins import SearchSlugManager, SearchSlugModelMixin
from edc_sites.model_mixins import SiteModelMixin

from ...model_mixins import VerifyModelMixin
from .manifest import Manifest


class ManifestItemManager(SearchSlugManager, models.Manager):
    def get_by_natural_key(self, identifier, manifest_identifier):
        return self.get(identifier=identifier, manifest_identifier=manifest_identifier)


class ManifestItem(SiteModelMixin, SearchSlugModelMixin, VerifyModelMixin, BaseUuidModel):
    def get_search_slug_fields(self):
        return "identifier", "human_readable_identifier"

    manifest = models.ForeignKey(Manifest, on_delete=PROTECT)

    identifier = models.CharField(max_length=25)

    comment = models.CharField(max_length=25, default="", blank=True)

    objects = ManifestItemManager()

    def natural_key(self):
        return self.identifier, *self.manifest.natural_key()

    natural_key.dependencies = ("edc_lab.manifest", "sites.Site")

    @property
    def human_readable_identifier(self):
        x = self.identifier
        return f"{x[0:4]}-{x[4:8]}-{x[8:12]}"

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Manifest Item"
        constraints = (
            UniqueConstraint(
                fields=["manifest", "identifier"],
                name="%(app_label)s_%(class)s_manifest_uniq",
            ),
        )



from __future__ import annotations

import json

from clinicedc_constants import NULL_STRING
from django.contrib.sites.models import Site
from django.db import models


class SiteProfile(models.Model):
    id = models.BigAutoField(primary_key=True)

    country = models.CharField(max_length=250, default=NULL_STRING)

    country_code = models.CharField(max_length=15, default=NULL_STRING)

    languages = models.TextField(default=NULL_STRING)

    title = models.CharField(max_length=250, default=NULL_STRING)

    site = models.OneToOneField(Site, on_delete=models.PROTECT)

    def __str__(self):
        return f"{self.site.id}: {self.title}"

    def get_languages(self):
        return json.loads(self.languages)



from .pill_bottle_model_mixin import PillBottleModelMixin


class PillBottle(PillBottleModelMixin):
    class Meta(PillBottleModelMixin.Meta):
        verbose_name = "Pill Bottle"
        verbose_name_plural = "Pill Bottles"



from .add_to_storage_bin_view import AddToStorageBinView
from .allocate_to_subject_view import AllocateToSubjectView
from .celery_task_status_view import CeleryTaskStatusView
from .confirm_at_location_view import ConfirmaAtLocationView
from .confirm_stock_from_queryset_view import ConfirmStockFromQuerySetView
from .dispense_view import DispenseView
from .get_stock_transfers_view import get_stock_transfers_view
from .home_view import HomeView
from .move_to_storage_bin_view import MoveToStorageBinView
from .prepare_and_review_stock_request_view import PrepareAndReviewStockRequestView
from .print_labels_view import PrintLabelsView
from .print_stock_transfer_manifest_view import print_stock_transfer_manifest_view
from .print_stock_view import print_stock_view
from .transfer_stock_view import TransferStockView
